<?php
namespace cloud\apps\ipfw;

defined('PANALOG_APPDIR') or define ('PANALOG_APPDIR', '/usr/logd/www/app/ipfw');
defined('IPFW') or define ('IPFW', '/sbin/ipfw');
defined('IPFW_CONF') or define ('IPFW_CONF', $_SERVER["DOCUMENT_ROOT"] . '/app/ipfw/ipfw.conf');

function ipfw_get($id)
{
	if (!file_exists(IPFW_CONF))
		return false;

	$lists = file(IPFW_CONF);
	foreach($lists as $val) {
		list($vid, $vdesc, $vdir, $vintype, $vouttype, $vsrcip, $vsrcport, $vdstip, $vdstport, $vproto, $vact) = explode(' ', $val);
		if ($vid == "id=$id")
			return $val;
	}

	return false;
}

function ipfw_split($str)
{
	$ds = explode('=', trim($str));

	return $ds[1];
}

function ipfw_remove($id)
{
	$ipfw_conf_tmp = IPFW_CONF.".tmp";

	if (!file_exists(IPFW_CONF))
		return false;

	if (($fp = fopen($ipfw_conf_tmp, "w")) == false)
		return false;

	$count = 0;
	$lists = file(IPFW_CONF);
	foreach($lists as $val) {
		list($vid, $vdesc, $vdir, $vintype, $vouttype, $vsrcip, $vsrcport, $vdstip, $vdstport, $vproto, $vact) = explode(' ', $val);
		if ($vid == "id=$id")
			continue;
		fprintf($fp, $val);
		$count++;
	}

	if($count == 0) {
		$rule = "id=65534 desc=允许所有 dir=any intype=any outtype=me srcip=";
		$rule .= " srcport=0 dstip= dstport=0 proto=all act=allow sta=1\n";
		fprintf($fp, $rule);
	}
	fclose($fp);

	rename($ipfw_conf_tmp, IPFW_CONF);

	return true;
}

function select($data)
{
	global $user;
	
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

//	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			$result = array(
				'rows' => array(),
				'limit' => $optional['limit']
			);
			if(isset($data['draw'])) $result['draw'] = $data['draw'];
			return $result;
		}
//	}

	$act = '';
	if(isset($data['act']) && $data['act'] != '') {
		if(intval($data['act']))
			$act = 'deny';
		else
			$act = 'allow';
	}

	$keyword = '';
	if(isset($data['keyword']) && $data['keyword'] != '') 
		$keyword = trim($data['keyword']);

	$rows	= array();
	$sort	= array();

	if (!file_exists(IPFW_CONF)) {

		$cmd = "id=65534 desc=允许所有 dir=any intype=any outtype=me srcip=";
		$cmd .= " srcport=0 dstip= dstport=0 proto=all act=allow sta=1\n";
		file_put_contents(IPFW_CONF, $cmd, FILE_APPEND);

		/* 检测ipfw是否是开启状态 */
		unset($out);
		$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl status";
		exec($cmd, $out, $ret);
		if (trim($out[0]) == "enable") {
			$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl stop";
			exec($cmd, $out, $ret);
			$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl start";
			exec($cmd, $out, $ret);
		}
	}

	$lists = file(IPFW_CONF);
	foreach($lists as $val) {
		list($vid, $vdesc, $vdir, $vintype, $vouttype, $vsrcip, $vsrcport, $vdstip, $vdstport, $vproto, $vact) = explode(' ', $val);
		$wact = ipfw_split($vact);

		if($act != '' && $act != $wact)
			continue;
		if($keyword != '' && strpos($val, $keyword))
			continue;

		$wid = ipfw_split($vid);
		array_push($rows, array(
			"id"		=>$wid,
			"desc"		=>ipfw_split($vdesc),
			"dir"		=>ipfw_split($vdir),
			"intype"	=>ipfw_split($vintype),
			"outtype"	=>ipfw_split($vouttype),
			"srcip"		=>ipfw_split($vsrcip),
			"srcport"	=>ipfw_split($vsrcport),
			"dstip"		=>ipfw_split($vdstip),
			"dstport"	=>ipfw_split($vdstport),
			"proto"		=>ipfw_split($vproto),
			"act"		=>$wact,
			"in"		=>0,
			"out"		=>0
		));

		array_push($sort, $wid);
	}

	$cmd = IPFW . " -t show | awk '{print $1, $2, $3}'";
	unset($out);
	exec($cmd, $out, $ret);
	if($ret == 0) {
		foreach($out as $val) {
			list($id, $out, $in) = explode(' ', $val);
			foreach($rows as $k=>$r) {
				if ($r["id"] == $id) {
					$rows[$k]["out"] = $out;
					$rows[$k]["in"] = $in;
					break;
				}
			}
		}
	}

	array_multisort($sort, SORT_ASC, SORT_NUMERIC, $rows);

	
//	$result['page'] = intval($optional['offset'] / $optional['limit']);
	$result['limit'] = $optional['limit'];
	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function add($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	if(isset($data['id']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '策略ID只能为 1~65535 之间的整数。');
		return false;
	}
	$id = intval($data['id']);
	if($id <= 0 || $id > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '策略ID只能为 1~65535 之间的整数。');
		return false;
	}

	if(isset($data['dir'])) {
		$dir = trim($data['dir']);
		if(in_array($dir, array('any', 'in', 'out')) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '数据流向不正确。');
			return false;
		}
	}
	else
		$dir = 'any';

	if(isset($data['desc']))
		$desc = trim($data['desc']);
	else
		$desc = '';

	if(isset($data['intype'])) {
		$intype = trim($data['intype']);
		if(in_array($intype, array('any', 'me', 'ip')) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '源IP类型正确。');
			return false;
		}
	}
	else
		$dir = 'any';

	if(isset($data['outtype'])) {
		$outtype = trim($data['outtype']);
		if(in_array($outtype, array('any', 'me', 'ip')) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '目标IP类型正确。');
			return false;
		}
	}
	else
		$outtype = 'any';


	if(isset($data['srcip'])) 
		$srcip = trim($data['srcip']);
	else
		$srcip = '';

	if(isset($data['srcport'])) {
		$srcport = intval($data['srcport']);
		if($srcport < 0 || $srcport > 65535) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '源端口只能为 0~65535 之间的整数。');
			return false;
		}
	}
	else
		$srcport = 0;
	

	if(isset($data['dstip']))
		$dstip = trim($data['dstip']);
	else
		$dstip = '';

	if(isset($data['dstport'])) {
		$dstport = trim($data['dstport']);
		if($dstport < 0 || $dstport > 65535) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '目标端口只能为 0~65535 之间的整数。');
			return false;
		}
	}
	else
		$dstport = 0;

	if(isset($data['proto'])) {
		$proto = trim($data['proto']);
		if(in_array($proto, array('all', 'udp', 'tcp', 'icmp')) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '协议不正确。');
			return false;
		}
	}
	else
		$proto = 'all';
	
	if(isset($data['act'])) {
		$act = trim($data['act']);
		if(in_array($act, array('allow', 'deny')) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '动作不正确。');
			return false;
		}
	}
	else
		$act = '';
	

	if(\cloud\apps\work\project\project_enable(array())) {
		if(!\cloud\apps\work\project\in_work_user($optional['username'])) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}
	else
	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}


	if (ipfw_get($id) != false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '此编号已经存在。');
		return false;
	}
	
	cloud_insertlog($user->username, "添加IPFW规则：{$id}");	


	$rule = "id=$id desc=$desc dir=$dir intype=$intype outtype=$outtype srcip=$srcip";
	$rule .= " srcport=$srcport dstip=$dstip dstport=$dstport proto=$proto act=$act sta=1\n";
	file_put_contents(IPFW_CONF, $rule, FILE_APPEND);

	/* 检测ipfw是否是开启状态 */
	unset($out);
	$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl status";
	exec($cmd, $out, $ret);
	if (trim($out[0]) == "enable") {
		$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl stop";
		exec($cmd, $out, $ret);
		$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl start";
		exec($cmd, $out, $ret);
	}
	
	return true;
}

function save($data)
{
	global $nidb, $user;
	
	
//	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
//	}
	
	if(isset($data['oldid']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '源策略ID只能为 1~65535 之间的整数。');
		return false;
	}
	$oldid = intval($data['oldid']);
	if($oldid <= 0 || $oldid > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '源策略ID只能为 1~65535 之间的整数。');
		return false;
	}

	if(isset($data['id']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '策略ID只能为 1~65535 之间的整数。');
		return false;
	}
	$id = intval($data['id']);
	if($id <= 0 || $id > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '策略ID只能为 1~65535 之间的整数。');
		return false;
	}
	
	if (ipfw_get($oldid)) {
		ipfw_remove($oldid);
	}
	
	return add($data);
}

function enable($data)
{
	global $nidb, $user;
	
	$enable = 0;
	$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl status";
	exec($cmd, $out, $ret);
	if($ret == 0) {
		if (trim($out[0]) == "enable")
			$enable = 1;
	}
	unset($out);
	if(isset($data['enable']) == false) 
		return $enable;
	
	if(intval($data['enable'])) {
		if($enable)
			return true;
	}
	else {
		if($enable == 0)
			return true;
	}
	$enable = intval($data['enable']);

//	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
//	}
	
	cloud_insertlog($user->username, ($enable?"启用":"禁用") . "IPFW服务");	

	$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl stop";
	exec($cmd, $out, $ret);
	unset($out);
	if($enable) {
		$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl start";
		exec($cmd, $out, $ret);
		if($ret) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '启动防火墙服务失败！' . implode(' ', $out));
			return false;
		}
	}

	set_errmsg(MSG_LEVEL_ARG, __function__, '修改成功！');
	return true;
}

function remove($data)
{
	global $nidb, $user;

	if(isset($data['id']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '策略ID只能为 1~65535 之间的整数。');
		return false;
	}
	$id = intval($data['id']);
	if($id <= 0 || $id > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '策略ID只能为 1~65535 之间的整数。');
		return false;
	}
	
	if ($id >= 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '65535 策略不可操作。');
		return false;
	}

	if (ipfw_get($id) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '此策略不存在。');
		return false;
	}

	if (ipfw_remove($id) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '');
		return false;
	}

//	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
//	}

	cloud_insertlog($user->username, "删除IPFW规则：" . $id);

	/* 检测ipfw是否是开启状态 */
	unset($out);
	$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl status";
	exec($cmd, $out, $ret);
	if (trim($out[0]) == "enable") {
		$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl stop";
		exec($cmd, $out, $ret);
		$cmd = $_SERVER["DOCUMENT_ROOT"] . "/app/ipfw/appctrl start";
		exec($cmd, $out, $ret);
	}

	return true;
}
