<?php
namespace cloud\apps\zeropen\sdwan\iwan;


function select($data)
{
	global $nidb, $user;

	$result = array(
		'rows'	=> array(),
		'total' => 0,
	);

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	if(!is_supadmin($user->username))
		return $result;

	$where_str = "";
	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(ent.`name` like ? or bra.`name` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	$order_map = array(
		"ent_id",
		"bra_id",
		"sridx",
		"srid",
		"pop_id",
		"ent",
		"name",
		"ctime"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($op['dir'])) {
				if(in_array($op['column'], $order_map)) {
					array_push($order, "`" . $op['column'] . "` " . ($op['dir']=='desc'?'desc':'asc'));
				}
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `ent_id`';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_platform.sdwan_branch_iwan";
	$sql.= " $where_str";

	try {
		$sth = $nidb->prepare($sql);

		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result['total'] = $rows[0]->total;
	}

	$sql = "select * from cloud_platform.sdwan_branch_iwan";
	$sql.= " $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}


	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function add($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}


	if(isset($data['name']) == false || empty(($data['name'] = trim($data['name'])))) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}
	if(isset($data['sridx']) == false || empty(($data['sridx'] = trim($data['sridx'])))) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'SRIDX不能为空。');
		return false;
	}
	$data['sridx'] = intval($data['sridx']);
	if($data['sridx'] <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'SRIDX不能小于等于0。');
		return false;
	}


	$frmData = array(
		'name'		=> $data['name'],
		'sridx'		=> $data['sridx'],
		'ctime'		=> time()
	);

	try {
		if(insert_data('cloud_platform`.`sdwan_enterprise', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$id = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

    return $id;
}

function save($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$values = array();
	
	if(isset($data['ent_id']) == false || empty(($ent_id = trim($data['ent_id'])))) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '企业ID不能为空。');
		return false;
	}
	$ent_id = intval($ent_id);
	if($ent_id <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '企业ID不能小于等于0。');
		return false;
	}
	if(isset($data['name'])) {
		if(empty(($data['name'] = trim($data['name'])))) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '企业名称不能为空。');
			return false;
		}
		$values['name'] = $data['name'];
	}

	if(isset($data['sridx'])) {
		if(empty(($data['name'] = trim($data['name'])))) {
			set_errmsg(MSG_LEVEL_ARG, __function__, 'SRIDX不能为空。');
			return false;
		}

		$data['sridx'] = intval($data['sridx']);
		if($data['sridx'] <= 0) {
			set_errmsg(MSG_LEVEL_ARG, __function__, 'SRIDX不能小于等于0。');
			return false;
		}
		$values['sridx'] = $data['sridx'];
	}

	if(count($values) <= 0)
		return 0;

	// 检查对象
	try {
		$sql = "select * from `cloud_platform`.`sdwan_enterprise`";
		$sql.= " where `ent_id` = {$ent_id}";
		$sth = $nidb->prepare($sql);

		$sth->execute();
		// $cnt = $sth->rowCount();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的企业不存在。');
		return false;
	}

	$where = array(
		'ent_id'	=> $ent_id,
	);

	try {
		if(update_data('cloud_platform`.`sdwan_enterprise', $values, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function del($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	if(isset($data['ent_id']) == false || empty(($ent_id = trim($data['ent_id'])))) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '企业ID不能为空。');
		return false;
	}
	$ent_id = intval($ent_id);
	if($ent_id <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '企业ID不能小于等于0。');
		return false;
	}

	// 检查对象
	try {
		$sql = "select * from `cloud_platform`.`sdwan_enterprise`";
		$sql.= " where `ent_id` = {$ent_id}";
		$sth = $nidb->prepare($sql);

		$sth->execute();
		// $cnt = $sth->rowCount();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的企业不存在。');
		return false;
	}

	try {
		$sql = "delete from `cloud_platform`.`sdwan_branch_iwan` where `ent_id` = {$ent_id}";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();

		$sql = "delete from `cloud_platform`.`sdwan_branch` where `ent_id` = {$ent_id}";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
		
		$sql = "delete from `cloud_platform`.`sdwan_enterprise` where `ent_id` = {$ent_id}";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}
