<?php
namespace cloud\apps\topo\custom\config;


function select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';

	if(!is_supadmin($user->username)) {
		$where_str .= "`founder` = ? and ";
		array_push($values, $user->username);
	}

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`name` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	$order_map = array(
		"tid",
		"founder",
		"name",
		"update_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `tid` desc';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_platform.custom_topo $where_str";
	try {
		$sth = $nidb->prepare($sql);

		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '`tid`,
`founder`,
`name`,
`update_time`,
`create_time`';

	$sql = "select $keys from cloud_platform.custom_topo $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	
	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function info($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	
	$isadmin = is_supadmin($user->username);

	try {
		$sql = "select * from `cloud_platform`.`custom_topo` where `tid` = ?";
		if(!$isadmin)
			$sql.= " and `founder` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		if(!$isadmin)
			$sth->bindParam(2, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该配置不存在！');
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $row;
}

function add($data)
{
	global $nidb, $user;
	

	$optional = array(
		'founder'	=> $user->username
	);
	if(format_and_push($data, 'name', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}

	$optional['extend'] = '';
	
	$optional['update_time'] = time();
	$optional['create_time'] = $optional['update_time'];

	cloud_insertlog($user->username, "创建自定义拓扑图“{$optional['name']}”");

	try {
		if(insert_data('cloud_platform`.`custom_topo', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}

		return true;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	return false;
}

function remove($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	
	$isadmin = is_supadmin($user->username);

	try {
		$sql = "select * from `cloud_platform`.`custom_topo` where `tid` = ?";
		if(!$isadmin)
			$sql.= " and `founder` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		if(!$isadmin)
			$sth->bindParam(2, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该配置不存在！');
			return false;
		}
		
		// 删除自定义拓扑图
		$sql = "delete from `cloud_platform`.`custom_topo` where `tid` = ?";
		if(!$isadmin)
			$sql.= " and `founder` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		if(!$isadmin)
			$sth->bindParam(2, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		
		// 删除自定义拓扑图节点
		$sql = "delete from `cloud_platform`.`custom_topo_nodes` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		$sth->execute();
		
		// 删除自定义拓扑图统计组
		$sql = "delete from `cloud_platform`.`custom_topo_group` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		$sth->execute();
		
		// 删除自定义拓扑图统计组成员
		$sql = "delete from `cloud_platform`.`custom_topo_group_child` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		$sth->execute();

		// 删除自定义拓扑图统计节点关联
		$sql = "delete from `cloud_platform`.`custom_topo_nodes_links` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	cloud_insertlog($user->username, "删除自定义拓扑图“{$optional['name']}”");

	return true;
}

function save($data)
{
	global $nidb, $user;
	

	$where = array();
	if(format_and_push($data, 'tid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	$optional = array();
	format_and_push($data, 'name', $optional, '', 'string', false);
	
	if(count($optional) == 0) 
		return true;
	
	$optional['update_time'] = time();
	$isadmin = is_supadmin($user->username);

	try {
		$sql = "select * from `cloud_platform`.`custom_topo` where `tid` = ?";
		if(!$isadmin)
			$sql.= " and `founder` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['tid'], \PDO::PARAM_INT);
		if(!$isadmin)
			$sth->bindParam(2, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该配置不存在！');
			return false;
		}
		
		if($optional['name'] != $row['name']) {
			if(update_data('cloud_platform`.`custom_topo', $optional, $where) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				return false;
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if($optional['name'] != $row['name'])
		cloud_insertlog($user->username, "修改自定义拓扑图“{$row['name']}”改为“{$optional['name']}”");
	else
		cloud_insertlog($user->username, "修改自定义拓扑图“{$optional['name']}”");

	return true;
}

function import($data)
{
	global $nidb, $user;


	if(isset($data['id']) == false
	|| ($id = intval($data["id"])) < 0)
		$id = 0;

	if(isset($_FILES['file']) == false
	|| $_FILES["file"]["tmp_name"] == ''
	|| !file_exists($_FILES["file"]["tmp_name"])) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件为空。');
		return false;
	}

	$arr = explode('/', $_FILES["file"]['type']);
	if($arr[0] != 'application') {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件类型无效。');
		return false;
	}
/*
  ["type"] "application/json"
*/
	if(strpos($arr[1], 'json') === false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件必须是json类型。');
		return false;
	}

	$data = file_get_contents($_FILES["file"]["tmp_name"]);
	if(!$data) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上文的文件内容无效！');
		return false;
	}
	$data = json_decode($data, true);
	if(!$data) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '解析文件内容失败！');
		return false;
	}
	
	if(!isset($data['topo']) && !isset($data['image'])) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上文的文件内容为空！');
		return false;
	}

	$path = LOGDATA_DIR . "/cloud_image";
	$ret = array();
	if(isset($data['image'])) {
		// 导入图片
		// 检查并创建目录
		if(!is_dir($path)) {
			if(!mkdir($path)) {
				set_errmsg(MSG_LEVEL_ARG, __function__, '创建图片保存目录失败！' . $path);
				return false;
			}
		}
		$ret['image'] = array();
		foreach($data['image'] as $name => $val) {
			if(!file_put_contents("{$path}/{$name}", base64_decode($val))) {
				set_errmsg(MSG_LEVEL_ARG, __function__, '导入图片失败！' . $name);
				return false;
			}
			array_push($ret['image'], $name);
		}
	}

	if(isset($data['topo'])) {
		if(!isset($data['topo']['config'])) {
			return $ret;
		}

		$now = time();
		$ret['topo'] = array();

		$hash = array();
		$grphash = array();
		
		// 导入拓扑配置
		$config = $data['topo']['config'];
		if($id) {
			$where = array('tid' => $id);
			if(!($info = info($where))) 
				return false;
			
			$config['name'] = $info['name'];
			try {
				if(update_data('cloud_platform`.`custom_topo', $config, $where) === false) {
					$errmsg = implode(' ', $nidb->errorInfo());
					set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
					return false;
				}
			}
			catch (\NiDBException $e) {
				set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
				return false;
			}
			
			// 导入设备
			if(isset($data['topo']['nodes'])) {
				// 找出当前设备
				$ret['topo']['nodes'] = count($data['topo']['nodes']);
				try {
					$sql = "select `nid` from `cloud_platform`.`custom_topo_nodes` where `tid` = ?";

					$sth = $nidb->prepare($sql);
					$sth->bindParam(1, $id, \PDO::PARAM_INT);
					$sth->execute();
					$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);

					$nodescnt = count($data['topo']['nodes']);
					if(count($rows) >= $nodescnt) {
						// 更新已有的设备
						$where = array('tid' => $id);
						for($i = 0; $i < $nodescnt; $i++) {
							$where['nid'] = $rows[$i]['nid'];
							//$data['topo']['nodes'][$i]['incnt'] = 0;
							//$data['topo']['nodes'][$i]['outcnt'] = 0;
							$data['topo']['nodes'][$i]['update_time'] = $now;
							
							$hash['n' . $data['topo']['nodes'][$i]['nid']] = $rows[$i]['nid'];
							unset($data['topo']['nodes'][$i]['nid']);
							if(update_data('cloud_platform`.`custom_topo_nodes', $data['topo']['nodes'][$i], $where) === false) {
								$errmsg = implode(' ', $nidb->errorInfo());
								set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
								return false;
							}
						}
						
						// 删除多余的设备
						$ids = array();
						for($i = count($rows) - 1; $i >= $nodescnt; $i--) 
							array_push($ids, $rows[$i]['nid']);
						if(count($ids) > 0) {
							$sql = "delete from `cloud_platform`.`custom_topo_nodes` where `nid` in (" . implode(',', $ids) . ")";
							$sth = $nidb->prepare($sql);
							$sth->execute();
						}
					}
					else {
						// 添加设备
						for($i = count($rows); $i < $nodescnt; $i++) {
							//$data['topo']['nodes'][$i]['incnt'] = 0;
							//$data['topo']['nodes'][$i]['outcnt'] = 0;
							$data['topo']['nodes'][$i]['tid'] = $id;
							$data['topo']['nodes'][$i]['update_time'] = $now;
							$data['topo']['nodes'][$i]['create_time'] = $now;
							
							$oid = $data['topo']['nodes'][$i]['nid'];
							unset($data['topo']['nodes'][$i]['nid']);
							if(insert_data('cloud_platform`.`custom_topo_nodes', $data['topo']['nodes'][$i]) === false) {
								$errmsg = implode(' ', $nidb->errorInfo());
								set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
								return false;
							}
							
							$hash['n' . $oid] = $nidb->lastInsertId();
						}
					}
					
					// 删除自定义拓扑图统计节点关联
					$sql = "delete from `cloud_platform`.`custom_topo_nodes_links` where `tid` = ?";
					$sth = $nidb->prepare($sql);
					$sth->bindParam(1, $id, \PDO::PARAM_INT);
					$sth->execute();
				}
				catch (\NiDBException $e) {
					set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
					return false;
				}
			}
			else {
				// 删除自定义拓扑图统计组成员
				$sql = "delete from `cloud_platform`.`custom_topo_nodes` where `tid` = ?";
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $id, \PDO::PARAM_INT);
				$sth->execute();
				
				// 删除自定义拓扑图统计组成员
				$sql = "delete from `cloud_platform`.`custom_topo_nodes_links` where `tid` = ?";
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $id, \PDO::PARAM_INT);
				$sth->execute();
			}
			
			// 导入拓扑分类
			if(isset($data['topo']['group'])) {
				// 找出当前分类
				$ret['topo']['group'] = count($data['topo']['group']);
				try {
					$sql = "select `gid` from `cloud_platform`.`custom_topo_group` where `tid` = ?";

					$sth = $nidb->prepare($sql);
					$sth->bindParam(1, $id, \PDO::PARAM_INT);
					$sth->execute();
					$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);

					$grpcnt = count($data['topo']['group']);
					if(count($rows) >= $grpcnt) {
						// 更新已有的分类
						$where = array('tid' => $id);
						for($i = 0; $i < $grpcnt; $i++) {
							$where['gid'] = $rows[$i]['gid'];
							// $data['topo']['group'][$i]['count'] = 0;
							$data['topo']['group'][$i]['update_time'] = $now;
							
							$grphash['n' . $data['topo']['group'][$i]['gid']] = $rows[$i]['gid'];
							unset($data['topo']['group'][$i]['gid']);
							if(update_data('cloud_platform`.`custom_topo_group', $data['topo']['group'][$i], $where) === false) {
								$errmsg = implode(' ', $nidb->errorInfo());
								set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
								return false;
							}
						}
						
						// 删除多余的分类
						$ids = array();
						for($i = count($rows) - 1; $i >= $grpcnt; $i--) 
							array_push($ids, $rows[$i]['gid']);
						if(count($ids) > 0) {
							$sql = "delete from `cloud_platform`.`custom_topo_group` where `gid` in (" . implode(',', $ids) . ")";
							$sth = $nidb->prepare($sql);
							$sth->execute();
						}
					}
					else {
						// 添加分类
						for($i = count($rows); $i < $grpcnt; $i++) {
							$data['topo']['group'][$i]['tid'] = $id;
							//$data['topo']['group'][$i]['count'] = 0;
							$data['topo']['group'][$i]['update_time'] = $now;
							$data['topo']['group'][$i]['create_time'] = $now;
							
							$oid = $data['topo']['group'][$i]['gid'];
							unset($data['topo']['group'][$i]['gid']);
							if(insert_data('cloud_platform`.`custom_topo_group', $data['topo']['group'][$i]) === false) {
								$errmsg = implode(' ', $nidb->errorInfo());
								set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
								return false;
							}
							$grphash['n' . $oid] = $nidb->lastInsertId();
						}
					}

					// 删除自定义拓扑图统计组成员
					$sql = "delete from `cloud_platform`.`custom_topo_group_child` where `tid` = ?";
					$sth = $nidb->prepare($sql);
					$sth->bindParam(1, $id, \PDO::PARAM_INT);
					$sth->execute();

				}
				catch (\NiDBException $e) {
					set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
					return false;
				}
			}
			else {
				// 清空分类
			
				// 删除自定义拓扑图统计组成员
				$sql = "delete from `cloud_platform`.`custom_topo_group` where `tid` = ?";
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $id, \PDO::PARAM_INT);
				$sth->execute();
				
				// 删除自定义拓扑图统计组成员
				$sql = "delete from `cloud_platform`.`custom_topo_group_child` where `tid` = ?";
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $id, \PDO::PARAM_INT);
				$sth->execute();
			}

			cloud_insertlog($user->username, "导入自定义拓扑图“{$config['name']}”");
		}
		else {

			$config['founder'] = $user->username;
			$config['update_time'] = $now;
			$config['create_time'] = $config['update_time'];

			try {
				if(insert_data('cloud_platform`.`custom_topo', $config) === false) {
					$errmsg = implode(' ', $nidb->errorInfo());
					set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
					return false;
				}
				$id = $nidb->lastInsertId();
				
			
				// 导入拓扑设备
				if(isset($data['topo']['nodes'])) {
					$count = count($data['topo']['nodes']);
					for($i = 0; $i < $count; $i++) {
						$data['topo']['nodes'][$i]['tid'] = $id;
						//$data['topo']['nodes'][$i]['incnt'] = 0;
						//$data['topo']['nodes'][$i]['outcnt'] = 0;
						$data['topo']['nodes'][$i]['update_time'] = $now;
						$data['topo']['nodes'][$i]['create_time'] = $now;
						
						$oid = $data['topo']['nodes'][$i]['nid'];
						unset($data['topo']['nodes'][$i]['nid']);
						if(insert_data('cloud_platform`.`custom_topo_nodes', $data['topo']['nodes'][$i]) === false) {
							$errmsg = implode(' ', $nidb->errorInfo());
							set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
							return false;
						}
						
						$hash['n' . $oid] = $nidb->lastInsertId();
					}
				}
				
				// 导入拓扑分类
				if(isset($data['topo']['group'])) {
					$count = count($data['topo']['group']);
					for($i = 0; $i < $count; $i++) {
						$data['topo']['group'][$i]['tid'] = $id;
						//$data['topo']['group'][$i]['count'] = 0;
						$data['topo']['group'][$i]['update_time'] = $now;
						$data['topo']['group'][$i]['create_time'] = $now;
						
						$oid = $data['topo']['group'][$i]['gid'];
						unset($data['topo']['group'][$i]['gid']);
						if(insert_data('cloud_platform`.`custom_topo_group', $data['topo']['group'][$i]) === false) {
							$errmsg = implode(' ', $nidb->errorInfo());
							set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
							return false;
						}
						$grphash['n' . $oid] = $nidb->lastInsertId();
					}
				}
			}
			catch (\NiDBException $e) {
				set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
				return false;
			}

			cloud_insertlog($user->username, "导入自定义拓扑图“{$config['name']}”");
		}
		
		// 导入关系
		if(isset($data['topo']['links'])
		&& isset($data['topo']['nodes'])) {
			$ret['topo']['links'] = count($data['topo']['links']);
			
			foreach($data['topo']['links'] as $val) {
				if(isset($hash['n' . $val['src']])
				&& isset($hash['n' . $val['dst']])) {
					$link = array(
						'tid'	=> $id,
						'src'	=> $hash['n' . $val['src']],
						'dst'	=> $hash['n' . $val['dst']],
						'srcio'	=> $val['srcio'],
						'dstio'	=> $val['dstio'],
					);
					if(insert_data('cloud_platform`.`custom_topo_nodes_links', $link) === false) {
						$errmsg = implode(' ', $nidb->errorInfo());
						set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
						return false;
					}
				}
			}
		}
		
		// 导入分类
		if(isset($data['topo']['child'])
		&& isset($data['topo']['group'])
		&& isset($hash)) {
			$ret['topo']['child'] = count($data['topo']['child']);
			
			foreach($data['topo']['child'] as $val) {
				if(isset($grphash['n' . $val['gid']])
				&& isset($hash['n' . $val['nid']])) {
					$child = array(
						'tid'	=> $id,
						'gid'	=> $grphash['n' . $val['gid']],
						'nid'	=> $hash['n' . $val['nid']]
					);
					if(insert_data('cloud_platform`.`custom_topo_group_child', $child) === false) {
						$errmsg = implode(' ', $nidb->errorInfo());
						set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
						return false;
					}
				}
			}
		}

	
	}

	return $ret;
}

function export($data)
{
	global $nidb, $user;


	if(isset($data['id']) == false
	|| ($id = intval($data["id"])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定要导出的拓扑。');
		return false;
	}

	try {
		$key = "`name`, `extend`";
		$sql = "select {$key} from `cloud_platform`.`custom_topo` where `tid` = ?";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $id, \PDO::PARAM_STR);
		$sth->execute();
		$config = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$config) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '找不到拓扑！' . $id);
			return false;
		}
		
		$key = "`nid`, `name`, `type`, `bddevtype`, `bddevid`, `extend`, `incnt`, `outcnt`";
		$sql = "select {$key} from `cloud_platform`.`custom_topo_nodes` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $id, \PDO::PARAM_STR);
		$sth->execute();
		$nodes = $sth->fetchAll(\PDO::FETCH_ASSOC);

		$key = "`gid`, `name`, `index`, `count`, `extend`";
		$sql = "select {$key} from `cloud_platform`.`custom_topo_group` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $id, \PDO::PARAM_STR);
		$sth->execute();
		$group = $sth->fetchAll(\PDO::FETCH_ASSOC);

		$key = "`gid`, `nid`";
		$sql = "select {$key} from `cloud_platform`.`custom_topo_group_child` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $id, \PDO::PARAM_STR);
		$sth->execute();
		$child = $sth->fetchAll(\PDO::FETCH_ASSOC);

		$key = "`src`, `dst`, `srcio`, `dstio`";
		$sql = "select {$key} from `cloud_platform`.`custom_topo_nodes_links` where `tid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $id, \PDO::PARAM_STR);
		$sth->execute();
		$links = $sth->fetchAll(\PDO::FETCH_ASSOC);

		$images = array();

		foreach($nodes as $node) {
			if(!$node['extend'])
				continue;

			$extend = json_decode($node['extend'], true);
			if(!$extend)
				continue;

			if(!isset($extend['ico']) 
			|| !isset($extend['ico']['img'])
			|| $extend['ico']['img'] == '')
				continue;

			$imgfile = LOGDATA_DIR . "/cloud_image/" . $extend['ico']['img'];
			if(!isset($images[$extend['ico']['img']])) {
				$images[$extend['ico']['img']] = base64_encode(file_get_contents(LOGDATA_DIR . "/cloud_image/{$extend['ico']['img']}"));
			}
		}
		
		$topo = array(
			'config' => $config,
			'nodes'  => $nodes,
			'group'  => $group,
			'child'  => $child,
			'links'  => $links
		);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	cloud_insertlog($user->username, "导出自定义拓扑图“{$config['name']}”");

	return array('topo' => $topo, 'image' => $images);
}

function custom_main($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		$optional['tid'] = 0;
	}

	$isadmin = is_supadmin($user->username);
	
	$where = " `tid` = {$optional['tid']} and";
	if(!$isadmin)
		$where.= " `founder` = ? and";

	if($where != "")
		$where = "where " . substr($where, 0, -3);
	$where.= "limit 1";

	try {
		$sql = "select * from `cloud_platform`.`custom_topo` {$where}";

		$sth = $nidb->prepare($sql);
		if(!$isadmin)
			$sth->bindParam(1, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		$topo = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$topo) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '请创建一个拓扑！');
			return false;
		}
		
		echo '{"ret":0,"msg":"","data":{';
		echo '"config":' . json_encode($topo);
/*
		$sql = "select `gid`,`name` from `cloud_platform`.`custom_topo_group` where `tid` = ? limit 10";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $topo['tid'], \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if($row) 
			echo ',"group":' . json_encode($row);
		else
			echo ',"group":[],';

		$sql = "select `nid`, `name`, `type`, `extend` from `cloud_platform`.`custom_topo_nodes` where `tid` = ? limit 10";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $row['tid'], \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if($row) 
			echo ',"nodes":' . json_encode($row);
		else
			echo ',"nodes":[]';
*/
		echo '}}';
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}
