<?php
namespace cloud\apps\topo\custom\group;


function nodeinfo($nid)
{
	global $nidb, $user;

	try {
		$sql = "select * from `cloud_platform`.`custom_topo_nodes` where `nid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $nid, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '节点不存在！');
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $row;
}

function topoinfo($tid)
{
	global $nidb, $user;

	$isadmin = is_supadmin($user->username);
	
	try {
		$sql = "select * from `cloud_platform`.`custom_topo` where `tid` = ?";
		if(!$isadmin)
			$sql.= " and `founder` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $tid, \PDO::PARAM_INT);
		if(!$isadmin)
			$sth->bindParam(2, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该配置不存在！');
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $row;
}

function select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	$where_str .= "tid = ? and ";
	array_push($values, $optional['tid']);
	if(format_and_push($data, 'gid', $optional, '', 'int', false)) {
		$where_str .= "gid = ? and ";
		array_push($values, $optional['gid']);
	}
	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`name` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	$order_map = array(
		"gid",
		"tid",
		"name",
		"index",
		"count",
		"update_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `gid` desc';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_platform.custom_topo_group $where_str";
	try {
		$sth = $nidb->prepare($sql);

		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '`gid`,
`tid`,
`name`,
`index`,
`count`,
`update_time`,
`create_time`';

	$sql = "select $keys from cloud_platform.custom_topo_group $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	
	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function info($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'gid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '分类ID不能为空。');
		return false;
	}

	try {
		$sql = "select * from `cloud_platform`.`custom_topo_group` where `gid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['gid'], \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '分类不存在！');
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $row;
}

function add($data)
{
	global $nidb, $user;
	

	$optional = array();
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'name', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}
	format_and_push($data, 'index', $optional, '', 'int', false);

	if(($topo = topoinfo($optional['tid'])) === false)
		return false;

	$optional['extend'] = '';
	$optional['update_time'] = time();
	$optional['create_time'] = $optional['update_time'];

	cloud_insertlog($user->username, "创建自定义拓扑图分类“{$topo['name']}/{$optional['name']}”");

	try {
		if(insert_data('cloud_platform`.`custom_topo_group', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}

	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	return true;
}

function remove($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'gid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '分类ID不能为空。');
		return false;
	}

	if(($group = info($optional)) === false)
		return false;
	if(($topo = topoinfo($group['tid'])) === false)
		return false;

	try {
		// 删除自定义拓扑图节点
		$sql = "delete from `cloud_platform`.`custom_topo_group` where `gid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['gid'], \PDO::PARAM_INT);
		$sth->execute();
		
		// 删除自定义拓扑图统计分类成员
		$sql = "delete from `cloud_platform`.`custom_topo_group_child` where `gid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['gid'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	cloud_insertlog($user->username, "删除自定义拓扑图分类“{$topo['name']}/{$group['name']}”");

	return true;
}

function save($data)
{
	global $nidb, $user;


	$where = array();
	if(format_and_push($data, 'gid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '分类ID不能为空。');
		return false;
	}

	if(($group = info($where)) === false)
		return false;
	if(($topo = topoinfo($group['tid'])) === false)
		return false;

	$optional = array();
	format_and_push($data, 'name', $optional, '', 'string', false);
	format_and_push($data, 'index', $optional, '', 'int', false);
	if(count($optional) == 0) 
		return true;

	$optional['update_time'] = time();

	try {
		if($optional['name'] != $row['name']) {
			if(update_data('cloud_platform`.`custom_topo_group', $optional, $where) === false) {
				$errmsg = implode(' ', $nidb->errorInfo());
				set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
				return false;
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($optional['name'] != $row['name'])
		cloud_insertlog($user->username, "修改自定义拓扑图分类“{$topo['name']}/{$group['name']}”改为“{$topo['name']}/{$optional['name']}”");
	else
		cloud_insertlog($user->username, "修改自定义拓扑图分类“{$topo['name']}/{$group['name']}”");

	return true;
}

function summary_child($gid)
{
	global $nidb, $user;

	try {
		// 统计计数器
		$sql.= "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`custom_topo_group_child`";
		$sql.= " where `gid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $gid, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新计数器
			$sql = "update `cloud_platform`.`custom_topo_group`";
			$sql.= " set `count` = ?";
			$sql.= " where `gid` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $gid, \PDO::PARAM_INT);
			$sth->execute();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function fetch($data)
{
	global $nidb, $user;

	$optional = array();
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}

	$keys = '`gid`, `name`, `index`, `count`, `extend`';
	$sql = "select $keys from `cloud_platform`.`custom_topo_group` where `tid` = ? order by `index` asc";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function child_list($data)
{
	global $nidb, $user;

	$optional = array();
	if(format_and_push($data, 'gid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '分类ID不能为空。');
		return false;
	}

	$keys = 'c.`nid`, c.`gid`, c.`tid`, n.`name`';
	$sql = "select $keys from `cloud_platform`.`custom_topo_group_child` as `c`";
	$sql.= " left join `cloud_platform`.`custom_topo_nodes` as `n` on c.`nid` = n.`nid`";
	$sql.= " where c.`gid` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['gid'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function child_add($data)
{
	global $nidb, $user;
	

	$where = array();
	if(format_and_push($data, 'tid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'gid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '分类ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'nid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '节点ID不能为空。');
		return false;
	}

	if(($topo = topoinfo($where['tid'])) === false)
		return false;

	$count = 0;
	try {
		// 是否存在
		$sql.= "select `nid`";
		$sql.= " from `cloud_platform`.`custom_topo_group_child`";
		$sql.= " where `gid` = {$where['gid']} and `tid` = {$where['tid']} and `nid` = {$where['nid']}";
		$sth = $nidb->prepare($sql);
		if($sth->execute()) {
			$row = $sth->fetch(\PDO::FETCH_ASSOC);
			if(!$row) {
				// 添加自定义拓扑图统计分类成员
				$sql = "insert into `cloud_platform`.`custom_topo_group_child` (`gid`,`tid`,`nid`)";
				$sql.= " values ({$where['gid']},{$where['tid']},{$where['nid']})";
				$sth = $nidb->prepare($sql);
				$sth->execute();
				
				$count++;
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 是否统计
	if($count
	||(isset($data['__nosummary']) == false || (isset($data['__nosummary']) && intval($data['__nosummary']) == 0))) {
		summary_child($where['gid']);
	}

	return true;
}

function child_rmv($data)
{
	global $nidb, $user;


	$where = array();
	if(format_and_push($data, 'tid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'gid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '分类ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'nid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '节点ID不能为空。');
		return false;
	}

	if(($topo = topoinfo($where['tid'])) === false)
		return false;

	$count = 0;
	try {
		// 是否存在
		$sql.= "select `nid`";
		$sql.= " from `cloud_platform`.`custom_topo_group_child`";
		$sql.= " where `tid` = {$where['tid']} and `gid` = {$where['gid']} and `nid` = {$where['nid']}";
		$sth = $nidb->prepare($sql);
		if($sth->execute()) {
			$row = $sth->fetch(\PDO::FETCH_ASSOC);
			if($row) {
				// 删除自定义拓扑图统计分类成员
				$sql = "delete from `cloud_platform`.`custom_topo_group_child` where `tid` = ? and `gid` = ? and `nid` = ?";
				$sth = $nidb->prepare($sql);
				$sth->bindParam(1, $where['tid'], \PDO::PARAM_INT);
				$sth->bindParam(2, $where['gid'], \PDO::PARAM_INT);
				$sth->bindParam(3, $where['nid'], \PDO::PARAM_INT);
				$sth->execute();

				$count++;
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	// 是否统计
	if($count 
	|| (isset($data['__nosummary']) == false || (isset($data['__nosummary']) && intval($data['__nosummary']) == 0))) {
		summary_child($where['gid']);
	}
	
	return true;
}