<?php
namespace cloud\apps\topo\image;


function get_file($keyword, $offset, $limit)
{
	$ret = array();
	$total = 0;

	$path = LOGDATA_DIR . "/cloud_image";
	
	if (is_dir($path) 
	&& $dh = opendir($path)){
		if($keyword == "") {
			while (($file = readdir($dh)) !== false) {
				if($file != "." && $file != "..") {
					if (is_file($path . '/' . $file)) {
						if($total >= $offset
						&& $total < $limit)
							array_push($ret, array(
								'name' => $file,
								'size' => filesize($path . '/' . $file)
							));
						$total++;
					}
				}
			}
		}
		else {
			while (($file = readdir($dh)) !== false) {
				if($file != "." && $file != "..") {
					if (strpos($file, $keyword) !== false
					&& is_file($path . '/' . $file)) {
						if($total >= $offset
						&& $total < $limit)
							array_push($ret, array(
								'name' => $file,
								'size' => filesize($path . '/' . $file)
							));
						$total++;
					}
				}
			}
		}
		closedir($dh);
	}

	return array('rows' => $ret, 'total' => $total);
}

function select($data)
{
	global $nidb, $user;


	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	if(format_and_push($data, 'keyword', $optional, '', 'string', false) == false)
		$optional['keyword'] = '';

	$result = get_file($optional['keyword'], $optional['offset'], $optional['limit']);

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function add($data)
{
	global $user;

	$path = LOGDATA_DIR . "/cloud_image";

	if(isset($_FILES['file']) == false
	|| $_FILES["file"]["tmp_name"] == ''
	|| !file_exists($_FILES["file"]["tmp_name"])) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件为空。');
		return false;
	}
	
	if($_FILES["file"]["size"] > 61440) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件不能大于60K。');
		return false;
	}

	$arr = explode('/', $_FILES["file"]['type']);
	if($arr[0] != 'image') {
		set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件不是图片类型。');
		return false;
	}

	if(!is_dir($path)) {
		if(!mkdir($path)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '创建图片保存目录失败！' . $path);
			return false;
		}
	}
	
	if(isset($data['name'])) {
		$name = trim($data['name']);
		if($name == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '指定的图标无效！');
			return false;
		}
		
		$imgfile = "{$path}/{$name}";
		if(!file_exists($imgfile)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '指定要更换的图标不存在！');
			return false;
		}

		unlink($imgfile);
	}
	else {
		if($_FILES["file"]['name'] == '')
			$name = md5_file($_FILES["file"]["tmp_name"]);
		else
			$name = $_FILES["file"]['name'];
		
		$imgfile = "{$path}/{$name}";

		if(file_exists($imgfile)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '图片已存在。');
			return false;
		}
	}

	if(!move_uploaded_file($_FILES["file"]["tmp_name"], $imgfile)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '保存上传的图片失败！');
		return false;
	}	

	cloud_insertlog($user->username, "上传自定义拓扑图的设备图片“{$name}”");

	$img_dir = ROOT_DIR . "/assets/img/topo";
	if(!is_link($img_dir)) {
		exec("rm -rf {$img_dir} 2>&1", $err, $ret);
		unset($err);
		exec("ln -s {$path} {$img_dir} 2>&1", $err, $ret);
		if($ret) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '链接图片目录失败！' . "ln -s {$path} {$img_dir} " . implode(' ', $err));
			return false;
		}
	}

	return true;
}

function remove($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'name', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '图片名不能为空。');
		return false;
	}
	
	$path = LOGDATA_DIR . "/cloud_image";
	$imgfile = "{$path}/{$optional['name']}";
	
	if(file_exists($imgfile)) {
		unlink($imgfile);
		cloud_insertlog($user->username, "删除自定义拓扑图的设备图片“{$optional['name']}”");
	}

	return true;
}


function change_image($data)
{
	global $user;
	
	if(isset($data['nid']) == false
	|| ($nid = intval($data['nid'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}
	if(($node = info(array('nid' => $nid))) === false)
		return false;
	
	if($node['extend'])
		$extend = json_decode($node['extend'], true);
	if(!$extend) 
		$extend = array();
	
	if(isset($data['clear']) && intval($data['clear'])) {
		if(isset($extend['ico']))
			unset($extend['ico']);
	}
	else {
		var_dump($_FILES['file']);
		if(isset($_FILES['file']) == false
		|| $_FILES["file"]["tmp_name"] !== ''
		|| !file_exists($_FILES["file"]["tmp_name"])) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '上传文件为空。');
			return false;
		}

		if(($content = file_get_contents($_FILES["file"]["tmp_name"]))) {
			// move_uploaded_file($_FILES["file"]["tmp_name"], $name)
			set_errmsg(MSG_LEVEL_ARG, __function__, '读取上传的文件失败！');
			return false;
		}

		$extend['ico'] = 'data:image/jpg/png/gif;base64,' . chunk_split(base64_encode($content));
	}

	$optional['extend'] = json_encode($extend);

	$optional['update_time'] = time();

	try {
		if(update_data('cloud_platform`.`custom_topo_nodes', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}
