<?php
namespace cloud\apps\topo\custom\nodes;


function topoinfo($tid)
{
	global $nidb, $user;

	$isadmin = is_supadmin($user->username);
	
	try {
		$sql = "select * from `cloud_platform`.`custom_topo` where `tid` = ?";
		if(!$isadmin)
			$sql.= " and `founder` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $tid, \PDO::PARAM_INT);
		if(!$isadmin)
			$sth->bindParam(2, $user->username, \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该配置不存在！');
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $row;
}

function select($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	$where_str .= "tid = ? and ";
	array_push($values, $optional['tid']);
	if(format_and_push($data, 'nid', $optional, '', 'int', false)) {
		$where_str .= "nid = ? and ";
		array_push($values, $optional['nid']);
	}
	$operators = array('=','>','<','>=','<=');
	if(format_and_push($data, 'incnt', $optional, '', 'string', false)) {
		$operator = trim(substr($optional['incnt'], 0, 2));
		$where_str .= "incnt {$operator} ? and ";
		if(in_array($operator, $operators)) {
			$incnt = intval(substr($optional['incnt'], 2));
			$where_str .= "incnt {$operator} {$incnt} and ";
		}
	}
	if(format_and_push($data, 'outcnt', $optional, '', 'string', false)) {
		$operator = trim(substr($optional['outcnt'], 0, 2));
		$where_str .= "outcnt {$operator} ? and ";
		if(in_array($operator, $operators)) {
			$outcnt = intval(substr($optional['outcnt'], 2));
			$where_str .= "outcnt {$operator} {$outcnt} and ";
		}
	}
	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`name` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	$order_map = array(
		"nid",
		"tid",
		"name",
		"type",
		"bddevtype",
		"bddevid",
		"incnt",
		"outcnt",
		"update_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `nid` desc';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_platform.custom_topo_nodes $where_str";
	try {
		$sth = $nidb->prepare($sql);

		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '`nid`,
`tid`,
`name`,
`type`,
`bddevtype`,
`extend`,
`bddevid`,
`incnt`,
`outcnt`,
`update_time`,
`create_time`';

	$sql = "select $keys from cloud_platform.custom_topo_nodes $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	
	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function info($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'nid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}

	try {
		$sql = "select * from `cloud_platform`.`custom_topo_nodes` where `nid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['nid'], \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '设备不存在！');
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $row;
}

function add($data)
{
	global $nidb, $user;
	

	$optional = array();
	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'name', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}
	format_and_push($data, 'bddevtype', $optional, '', 'int', false);
	format_and_push($data, 'bddevid', $optional, '', 'string', false);
	format_and_push($data, 'type', $optional, '', 'string', false);
	format_and_push($data, 'img', $optional, '', 'string', false);
	
	if(($topo = topoinfo($optional['tid'])) === false)
		return false;

	$extend = array(
		'cards' => array()
	);
	if(isset($data['inif']) && ($inif = trim($data['inif'])) != '') {
		$inif = explode(',', $inif);
		foreach($inif as $if) {
			$if = explode('|', $if, 2);
			if($if && count($if) > 1)
				$extend['cards'][$if[0]] = array (
					'name' => $if[1],
					'dir' => 'in',
				);
			else
				$extend['cards'][$if[0]] = array (
					'name' => $if[0],
					'dir' => 'in',
				);
		}
	}
	if(isset($data['outif']) && ($outif = trim($data['outif'])) != '') {
		$outif = explode(',', $outif);
		foreach($outif as $if) {
			$if = explode('|', $if, 2);
			if($if && count($if) > 1)
				$extend['cards'][$if[0]] = array (
					'name' => $if[1],
					'dir' => 'out',
				);
			else
				$extend['cards'][$if[0]] = array (
					'name' => $if[0],
					'dir' => 'out',
				);
		}
	}

	if(isset($optional['img'])) {
		if(isset($extend['ico']))
			$extend['ico']['img'] = $optional['img'];
		else
			$extend['ico'] = array('img' => $optional['img']);
	}
	
	$optional['extend'] = json_encode($extend);
	$optional['update_time'] = time();
	$optional['create_time'] = $optional['update_time'];

	cloud_insertlog($user->username, "创建自定义拓扑图设备“{$topo['name']}/{$optional['name']}”");

	try {
		if(insert_data('cloud_platform`.`custom_topo_nodes', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}

		return true;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	return false;
}

function summary_child($gid)
{
	global $nidb, $user;

	try {
		// 统计计数器
		$sql.= "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`custom_topo_group_child`";
		$sql.= " where `gid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $gid, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新计数器
			$sql.= "update `cloud_platform`.`custom_topo_group`";
			$sql.= " set `count` = ?";
			$sql.= " where `gid` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $gid, \PDO::PARAM_INT);
			$sth->execute();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function remove($data)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'nid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}

	if(($node = info($optional)) === false)
		return false;
	if(($topo = topoinfo($node['tid'])) === false)
		return false;

	try {
		// 找出哪些分类需要重新统计
		$sql = "select `gid`";
		$sql.= " from `cloud_platform`.`custom_topo_group_child`";
		$sql.= " where `nid` = ? group by `gid`";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['nid'], \PDO::PARAM_INT);
		$sth->execute();
		$gids = $sth->fetchAll(\PDO::FETCH_ASSOC);

		
		// 删除自定义拓扑图设备
		$sql = "delete from `cloud_platform`.`custom_topo_nodes` where `nid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['nid'], \PDO::PARAM_INT);
		$sth->execute();
		
		// 删除自定义拓扑图统计组成员
		$sql = "delete from `cloud_platform`.`custom_topo_group_child` where `nid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['nid'], \PDO::PARAM_INT);
		$sth->execute();

		// 删除自定义拓扑图统计设备关联
		$sql = "delete from `cloud_platform`.`custom_topo_nodes_links` where `src` = ? or `dst` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['nid'], \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['nid'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	cloud_insertlog($user->username, "删除自定义拓扑图设备“{$topo['name']}/{$node['name']}”");

	// 是否统计
	if($gids
	||(isset($data['__nosummary']) == false || (isset($data['__nosummary']) && intval($data['__nosummary']) == 0))) {
		foreach($gids as $gid)
			summary_child($gid);

		summary_inlinks($optional['nid']);
	}

	return true;
}

function position($data)
{
	global $nidb, $user;
	

	$optional = array();
	if(preg_match("/^([0-9-.]{1,})$/", $data['vx'], $match) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'vx坐标不正确！');
		return false;
	}
	if(preg_match("/^([0-9-.]{1,})$/", $data['vy'], $match) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'vy坐标不正确！');
		return false;
	}

	$where = array();
	if(format_and_push($data, 'nid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}

	if(($node = info($where)) === false)
		return false;
	if(($topo = topoinfo($node['tid'])) === false)
		return false;

	if($node['extend'])
		$extend = json_decode($node['extend'], true);
	if(!$extend) 
		$extend = array();

	$extend['vx'] = $data['vx'];
	$extend['vy'] = $data['vy'];
	$optional['extend'] = json_encode($extend);

	$optional['update_time'] = time();

	try {
		if(update_data('cloud_platform`.`custom_topo_nodes', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function save($data)
{
	global $nidb, $user;
	

	$optional = array();

	format_and_push($data, 'name', $optional, '', 'string', false);
	format_and_push($data, 'bddevtype', $optional, '', 'int', false);
	format_and_push($data, 'bddevid', $optional, '', 'string', false);
	format_and_push($data, 'type', $optional, '', 'string', false);
	format_and_push($data, 'img', $optional, '', 'string', false);
	if(count($optional) == 0)
		return true;

	$where = array();
	if(format_and_push($data, 'nid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}

	if(($node = info($where)) === false)
		return false;
	if(($topo = topoinfo($node['tid'])) === false)
		return false;

	if($node['extend'])
		$extend = json_decode($node['extend'], true);

	if(!$extend) {
		$extend = array();
	}

	$cards = array();

	if(isset($data['inif']) && ($inif = trim($data['inif'])) != '') {
		$havinif = 1;
		$inif = explode(',', $inif);
		foreach($inif as $if) {
			$if = explode('|', $if, 2);
			if($if && count($if) > 1)
				$cards[$if[0]] = array (
					'name' => $if[1],
					'dir' => 'in',
				);
			else
				$cards[$if[0]] = array (
					'name' => $if[0],
					'dir' => 'in',
				);
		}
	}
	if(isset($data['outif']) && ($outif = trim($data['outif'])) != '') {
		$havinif = 1;
		$outif = explode(',', $outif);
		foreach($outif as $if) {
			$if = explode('|', $if, 2);
			if($if && count($if) > 1)
				$cards[$if[0]] = array (
					'name' => $if[1],
					'dir' => 'out',
				);
			else
				$cards[$if[0]] = array (
					'name' => $if[0],
					'dir' => 'out',
				);
		}
	}
	
	if($havinif)
		$extend['cards'] = $cards;

	if(isset($optional['img'])) {
		if(isset($extend['ico']))
			$extend['ico']['img'] = $optional['img'];
		else
			$extend['ico'] = array('img' => $optional['img']);
	}

	$optional['extend'] = json_encode($extend);
	$optional['update_time'] = time();

	try {
		if(update_data('cloud_platform`.`custom_topo_nodes', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(isset($optional['name']) && $optional['name'] != $row['name'])
		cloud_insertlog($user->username, "修改自定义拓扑图设备“{$topo['name']}/{$node['name']}”改为“{$topo['name']}/{$optional['name']}”");
	else
		cloud_insertlog($user->username, "修改自定义拓扑图设备“{$topo['name']}/{$node['name']}”");

	return true;
}

function summary_inlinks($nid)
{
	global $nidb, $user;

	try {
		// 统计计数器
		$sql = "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`custom_topo_nodes_links`";
		$sql.= " where `dst` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $nid, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新计数器
			$sql = "update `cloud_platform`.`custom_topo_nodes`";
			$sql.= " set `incnt` = ?";
			$sql.= " where `nid` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $nid, \PDO::PARAM_INT);
			$sth->execute();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function summary_outlinks($nid)
{
	global $nidb, $user;

	try {
		// 统计计数器
		$sql = "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`custom_topo_nodes_links`";
		$sql.= " where `src` = ? group by `src`";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $nid, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新计数器
			$sql = "update `cloud_platform`.`custom_topo_nodes`";
			$sql.= " set `outcnt` = ?";
			$sql.= " where `nid` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $nid, \PDO::PARAM_INT);
			$sth->execute();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function fetch($data)
{
	global $nidb, $user;

	$values = array();
	$optional = array();
	// set order, limit, offset value

	if(format_and_push($data, 'tid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '拓扑ID不能为空。');
		return false;
	}
	$where_str = 'where `tid` = ?';
	array_push($values, $optional['tid']);
/*
	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= " and (`name` like ?)";
		array_push($values, "%" . $optional['keyword'] . "%");
	}
*/
	echo '{"ret":0,"msg":"","data":{';
	
	try {
		// nodes 
		$keys = '`nid`, `name`, `type`, `incnt`, `outcnt`, `bddevtype`, `bddevid`, `extend`';
		$sql = "select $keys from cloud_platform.custom_topo_nodes $where_str limit 4000";

		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);

		if($rows && count($rows) > 0) 
			echo '"nodes":' . json_encode($rows);
		else
			echo '"nodes":[]';

		// links
		$keys = '`src`, `dst`, `srcio`, `dstio`';
		$sql = "select $keys from cloud_platform.custom_topo_nodes_links where `tid` = ?";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tid'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);

		if($rows && count($rows) > 0) 
			echo ',"links":' . json_encode($rows);
		else
			echo ',"links":[]';
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	echo '}}';

	return true;
}

function change_image($data)
{
	global $nidb, $user;
	
	if(isset($data['img']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备图标不能为空。');
		return false;
	}
	$img = trim($data['img']);
	if(isset($data['nid']) == false
	|| ($nid = intval($data['nid'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}
	if(($node = info(array('nid' => $nid))) === false)
		return false;
	
	if($node['extend'])
		$extend = json_decode($node['extend'], true);
	if(!$extend) 
		$extend = array();
	
	if($img == '默认图标')
		$extend['ico']['img'] = '';
	else {
		if(isset($extend['ico']))
			$extend['ico']['img'] = $img;
		else
			$extend['ico'] = array('img' => $img);
	}

	$optional['extend'] = json_encode($extend);

	$optional['update_time'] = time();

	$where = array('nid' => $nid);
	try {
		if(update_data('cloud_platform`.`custom_topo_nodes', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function dump_binddevice($data)
{
	global $nidb, $user;


	if(preg_match("/^([a-zA-Z0-9-_.,]{1,})$/", $data['act'], $match) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '获取信息类型不正确！');
		return false;
	}

	$act = $data['act'];
	if(!isset($data['ids']) || ($data['ids'] = trim($data['ids'])) == '') {
		set_errmsg(MSG_LEVEL_ARG, __function__, '获取信息类型不正确！');
		return false;
	}
	
	$ret = array();
	$idsarr = array();
	$ids = explode(',', $data['ids']);
	foreach($ids as $id) {
		$id = intval($id);
		if($id > 0)
			array_push($idsarr, $id);
	}
	
	try {
		$sql = "select `nid`, `bddevtype`, `bddevid` from `cloud_platform`.`custom_topo_nodes` where `nid` in (" . implode(',', $idsarr) . ")";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['nid'], \PDO::PARAM_INT);
		$sth->execute();
		$rows= $sth->fetchAll(\PDO::FETCH_ASSOC);
		if(!$rows || count($rows) == 0) {
			return array();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	$cache = array();
	include_once(__DIR__ . '/panabit.php');
	foreach($rows as $info) {
		// 派网网关设备 bddevtype=1, bddevid=PN800A0713aH
		if($info['bddevtype'] == 1
		&& $info['bddevid']) {
			if($act == 'status') {
				if(preg_match("/^([a-zA-Z0-9-_,]{12,})$/", $info['bddevid'], $match) == false)
					continue;
				
				$serialno = $info['bddevid'];
				if(!isset($cache[$serialno]))
					$cache[$serialno] = \cloud\apps\topo\custom\dump\getdevice($info['bddevid']);
				if($cache[$serialno]) {
					array_push($ret, array(
						'nid' => $info['nid'],
						'data'=> $cache[$serialno]
					));
				}
			}
		}
	}

	return $ret;
}

function link_add($data)
{
	global $nidb, $user;


	$where = array();
	if(format_and_push($data, 'src', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '原设备ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'dst', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标点ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'srcio', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '原IO不能为空。');
		return false;
	}
	if(format_and_push($data, 'dstio', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标IO不能为空。');
		return false;
	}
	
	if($where['src'] == $where['dst'] && $where['srcio'] == $where['dstio']) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '不能创建自己连自己的IO连接。');
		return false;
	}

	if(($src = info(array('nid' => $where['src']))) === false)
		return false;
	if(($dst = info(array('nid' => $where['dst']))) === false)
		return false;
	if($src['tid'] != $dst['tid']) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备拓扑不匹配。');
		return false;
	}
	if(($topo = topoinfo($src['tid'])) === false)
		return false;

	try {
		// 添加自定义拓扑图设备关系
		$sql = "insert into `cloud_platform`.`custom_topo_nodes_links` (`tid`,`src`,`dst`,`srcio`,`dstio`)";
		$sql.= " values (?,?,?,?,?)";
		$sql.= " on duplicate key update `tid` = ?";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $src['tid'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['src'], \PDO::PARAM_INT);
		$sth->bindParam(3, $where['dst'], \PDO::PARAM_INT);
		$sth->bindParam(4, $where['srcio'], \PDO::PARAM_STR);
		$sth->bindParam(5, $where['dstio'], \PDO::PARAM_STR);
		$sth->bindParam(6, $src['tid'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 是否统计
	if(isset($data['__nosummary']) == false 
	||(isset($data['__nosummary']) && intval($data['__nosummary']) == 0)) {
		summary_inlinks($where['dst']);
		summary_outlinks($where['src']);
	}
	
	return true;
}

function link_rmv($data)
{
	global $nidb, $user;


	$where = array();
	if(format_and_push($data, 'src', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '原设备ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'dst', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标点ID不能为空。');
		return false;
	}
	if(format_and_push($data, 'srcio', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '原设备接口不能为空。');
		return false;
	}
	if(format_and_push($data, 'dstio', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标点接口不能为空。');
		return false;
	}
	
	if(($src = info(array('nid' => $where['src']))) === false)
		return false;
	if(($dst = info(array('nid' => $where['dst']))) === false)
		return false;
	if($src['tid'] != $dst['tid']) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备拓扑不匹配。');
		return false;
	}
	if(($topo = topoinfo($src['tid'])) === false)
		return false;

	try {
		// 删除自定义拓扑图设备关系
		$sql = "delete from `cloud_platform`.`custom_topo_nodes_links` where `src` = ? and `dst` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['src'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['dst'], \PDO::PARAM_INT);
		$sth->bindParam(3, $where['srcio'], \PDO::PARAM_STR);
		$sth->bindParam(4, $where['dstio'], \PDO::PARAM_STR);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 是否统计
	if(isset($data['__nosummary']) == false 
	||(isset($data['__nosummary']) && intval($data['__nosummary']) == 0)) {
		summary_inlinks($where['dst']);
		summary_outlinks($where['src']);
	}

	return true;
}


function link_get($data)
{
	global $nidb, $user;


	$where = array();
	if(format_and_push($data, 'nid', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '设备ID不能为空。');
		return false;
	}

	if(($node = info($where)) === false)
		return false;
	if(($topo = topoinfo($node['tid'])) === false)
		return false;

	try {
		// 删除自定义拓扑图设备关系
		$sql = "select `src`,`dst`,`srcio`,`dstio` from `cloud_platform`.`custom_topo_nodes_links` where `src` = ? or `dst` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['nid'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['nid'], \PDO::PARAM_INT);
		$sth->execute();
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $rows;
}
