<?php
namespace cloud\apps\tunnel\online;


function select($data)
{
	global $nidb, $user;


	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false)) {
		$where_str .= "`tp_id` = ? and ";
		array_push($values, $optional['tp_id']);
	}

	if(!is_supadmin($user->username)) {
		$where_str .= "`user` = ? and ";
		array_push($values, $user->username);
	}
	else {
		if(format_and_push($data, 'user', $optional, '', 'string', false)) {
			$where_str .= "`user` = ? and ";
			array_push($values, $optional['user']);
		}
	}
	if(format_and_push($data, 'target_host', $optional, '', 'string', false)) {
		$where_str .= "`target_host` = ? and ";
		array_push($values, $optional['target_host']);
	}
	if(format_and_push($data, 'target_port', $optional, '', 'int', false)) {
		$where_str .= "`target_port` = ? and ";
		array_push($values, $optional['target_port']);
	}

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`name` like ? or `user` like ? or `target_host` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$order_map = array(
		"tp_id",
		"name",
		"user",
		"port",
		"target_port",
		"target_host",
		"last_ip",
		"start_time",
		"last_time",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `create_time` desc';

	$sql = "select count(*) as `total` from cloud_platform.tunnel_online $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		return array(
			'total' => 0,
			'rows'	=> array()
		);
	}

	$sql = "select * from cloud_platform.tunnel_online $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	ni_app_load('webssh', 'webssh');
	$webssh_port = \cloud\apps\webssh\getport();
	$openid_dir = LOGDATA_DIR . "/App/webssh/openid";
	foreach($rows as $key => $row) {
		$openid = 'TU-' . md5('tunnel_' . $row->port);
		if(file_exists("{$openid_dir}/{$openid}.dat")) {
			$rows[$key]->openid = $openid;
			$rows[$key]->webssh_port = $webssh_port;
		}
	}
	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}


function add($data)
{
	global $nidb, $user;


	$where = array();
	$optional = array();
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定隧道。');
		return false;
	}

	if($user) {
		if(format_and_push($data, 'user', $optional, '', 'string', false) == false)
			$optional['user'] = $user->username;

		$isadmin = 0;
		if(!is_supadmin($user->username))
			$optional['user'] = $user->username;
		else
		if(!isset($data['user']))
			$isadmin = 1;

		$sql = "select * from cloud_platform.tunnel_user where `tp_id` = ? and `user` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
			$sth->bindParam(2, $optional['user'], \PDO::PARAM_STR);
			$sth->execute();
			$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if($isadmin == 0 && !$uinfo) {
			set_errmsg(MSG_LEVEL_EXP, __function__, "无效的访问！1");
			return false;
		}

		if($uinfo 
		&& $uinfo['expire_time'] && $uinfo['expire_time'] < time()) {
			set_errmsg(MSG_LEVEL_EXP, __function__, "访问已过期！");
			return false;
		}
	}
	else {
		if(format_and_push($data, 'user', $optional, '', 'string', false) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, 'user不能为空。');
			return false;
		}

		if(format_and_push($data, 'token', $where, '', 'string', false) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, 'token不能为空。');
			return false;
		}

		$sql = "select * from cloud_platform.tunnel_user where `tp_id` = ? and `user` = ? and `token` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
			$sth->bindParam(2, $optional['user'], \PDO::PARAM_STR);
			$sth->bindParam(3, $optional['token'], \PDO::PARAM_STR);
			$sth->execute();
			$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if(!$uinfo) {
			set_errmsg(MSG_LEVEL_EXP, __function__, "无效的访问！");
			return false;
		}

		if($uinfo['expire_time'] 
		&& $uinfo['expire_time'] < time()) {
			set_errmsg(MSG_LEVEL_EXP, __function__, "访问已过期！");
			return false;
		}
	}

	$sql = "select * from cloud_platform.tunnel_proxy where `tp_id` = ?";
	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
		$sth->execute();
		$proxy = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$proxy) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "无效的资源！");
		return false;
	}

	
	require_once(__DIR__ . '/tunnel_sshproxy.php');

	$optional['port'] = \cloud\apps\tunnel\sshproxy\open_sshproxy($proxy['target_host'], $proxy['target_port']);
	if($optional['port'] == 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "打开“{$proxy['name']}”资源失败！");
		return false;
	}
	$optional['pid'] = \cloud\apps\tunnel\sshproxy\get_sshproxy_pid($optional['port']);
	if($optional['pid'] == 0) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "打开“{$proxy['name']}”资源失败2！");
		return false;
	}

	$optional['port'] = 0;
	$optional['domain'] = $proxy['domain'];
	$optional['type'] = $proxy['type'];
	$optional['path'] = $proxy['path'];
	$optional['last_ip'] = get_real_ip();
	$optional['start_time'] = 0;
	$optional['last_time'] = time();
	$optional['create_time'] = $optional['last_time'];

	try {
		// 添加在线用户
		// `tp_id`, `user`, `port`, `last_ip`
		$sql = "insert into `cloud_platform`.`tunnel_online`";
		$sql.= " (`tp_id`,`name`,`user`,`pid`,`port`,`target_port`,`target_host`,`last_ip`,`start_time`,`last_time`,`create_time`,`path`,`type`,`domain`)";
		$sql.= " values (?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
		$sql.= " on duplicate key update `name` = ?, `target_port` = ?";
		$sql.= ", `target_host` = ?, `last_time` = ?";
		$sql.= ", `pid` = ?, `path` = ?, `type` = ?, `domain` = ?";

		$sth = $nidb->prepare($sql);
		// insert
		$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $proxy['name'], \PDO::PARAM_STR);
		$sth->bindParam(3, $optional['user'], \PDO::PARAM_STR);
		$sth->bindParam(4, $optional['pid'], \PDO::PARAM_INT);
		$sth->bindParam(5, $optional['port'], \PDO::PARAM_INT);
		$sth->bindParam(6, $proxy['target_port'], \PDO::PARAM_INT);
		$sth->bindParam(7, $proxy['target_host'], \PDO::PARAM_STR);
		$sth->bindParam(8, $optional['last_ip'], \PDO::PARAM_STR);
		$sth->bindParam(9, $optional['start_time'], \PDO::PARAM_INT);
		$sth->bindParam(10, $optional['last_time'], \PDO::PARAM_INT);
		$sth->bindParam(11, $optional['create_time'], \PDO::PARAM_INT);
		$sth->bindParam(12, $optional['path'], \PDO::PARAM_STR);
		$sth->bindParam(13, $optional['type'], \PDO::PARAM_INT);
		$sth->bindParam(14, $optional['domain'], \PDO::PARAM_STR);
		// update
		$sth->bindParam(15, $proxy['name'], \PDO::PARAM_STR);
		$sth->bindParam(16, $proxy['target_port'], \PDO::PARAM_INT);
		$sth->bindParam(17, $proxy['target_host'], \PDO::PARAM_STR);
		$sth->bindParam(18, $optional['last_time'], \PDO::PARAM_INT);
		$sth->bindParam(19, $optional['pid'], \PDO::PARAM_INT);
		$sth->bindParam(20, $optional['path'], \PDO::PARAM_STR);
		$sth->bindParam(21, $optional['type'], \PDO::PARAM_INT);
		$sth->bindParam(22, $optional['domain'], \PDO::PARAM_STR);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	summary_session($optional['tp_id'], $optional['user']);
	
	if($user) {
		cloud_insertlog($user->username, "创建隧道会话：帐号={$optional['user']}，主机={$proxy['name']}");
		$username = $user->username;
	} else {
		cloud_insertlog($optional['user'], "创建隧道会话：帐号={$optional['user']}，主机={$proxy['name']}");
		$username = $optional['user'];
	}
	
	// if ssh then create webssh openid
	if($optional['type'] == 3
	&& file_exists('/usr/logd/App/webssh/appctrl')) {
		$localip = $_SERVER['HTTP_HOST'];
		if (strchr($localip, ':') != false) {
			$ds = explode(':', $localip);
			$localip = $ds[0];
		}

		$openid = 'TU-' . md5('tunnel_' . $optional['port']);
		$optional['openid'] = $openid;


		if($optional['path'] == '')
			$target_host = $localip;
		else
			$target_host = "{$optional['path']}@{$localip}";

		ni_app_load('webssh', 'webssh');
		\cloud\apps\webssh\add(array(
			'openid'	=> $openid,
			'target_port'=> $optional['port'],
			'target_host'=> "{$target_host}",
			'user'		=> $username,
			'titile'	=> $proxy['name'],
			'name'		=> $proxy['name'],
			'serialno'	=> $proxy['name'],
			'createtm'	=> $optional['last_time']
		));

		$optional['webssh_port'] = \cloud\apps\webssh\getport();
	}
	
	return $optional;
}

function remove($data)
{
	global $nidb, $user;


	$where = array();
	$optional = array();
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定隧道。');
		return false;
	}
	if(format_and_push($data, 'pid', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, 'PID不能为空。');
		return false;
	}

	// `tp_id`, `user`, `pid`
	if(!is_supadmin($user->username)) {
		$optional['user'] = $user->username;

		$sql = "select * from cloud_platform.tunnel_online where `tp_id` = ? and `user` = ? and `pid` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
			$sth->bindParam(2, $optional['user'], \PDO::PARAM_STR);
			$sth->bindParam(3, $optional['pid'], \PDO::PARAM_INT);
			$sth->execute();
			$online = $sth->fetch(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}
	else {
		$sql = "select * from cloud_platform.tunnel_online where `tp_id` = ? and `pid` = ?";
		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
			$sth->bindParam(2, $optional['pid'], \PDO::PARAM_INT);
			$sth->execute();
			$online = $sth->fetch(\PDO::FETCH_ASSOC);
		}
		catch (\NiDBException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}
	}

	// 关闭进程
	if(!$online) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "无效的资源！");
		return false;
	}

	$cmd = "kill {$online['pid']} > /dev/null 2>&1";
	exec($cmd, $out, $ret);
	unset($out);

	try {
		$sql = "delete from cloud_platform.tunnel_online where `tp_id` = ? and `pid` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['pid'], \PDO::PARAM_INT);
		$sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	summary_session($where['tp_id'], $online['user']);

	cloud_insertlog($user->username, "删除隧道会话：会话={$optional['pid']}，帐号={$online['user']}，主机={$online['name']}");

	return true;
}

function summary_session($tp_id, $user)
{
	global $nidb;

	try {
		// 统计在线会话
		$sql = "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`tunnel_online`";
		$sql.= " where `tp_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $tp_id, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新隧道会话计数器
			$sql = "update `cloud_platform`.`tunnel_proxy`";
			$sql.= " set `count` = ?";
			$sql.= " where `tp_id` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $tp_id, \PDO::PARAM_INT);
			$sth->execute();
		}

		// 统计在线帐号隧道的会话数
		$sql = "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`tunnel_online`";
		$sql.= " where `tp_id` = ? and `user` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $tp_id, \PDO::PARAM_INT);
		$sth->bindParam(2, $user, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新帐号隧道的计数器
			$sql = "update `cloud_platform`.`tunnel_user`";
			$sql.= " set `count` = ?";
			$sql.= " where `tp_id` = ? and `user` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $tp_id, \PDO::PARAM_INT);
			$sth->bindParam(3, $user, \PDO::PARAM_INT);
			$sth->execute();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

