<?php
namespace cloud\apps\tunnel\proxy;


function select($data)
{
	global $nidb, $user;


	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	if(!is_supadmin($user->username)) {
		return array(
			'total' => 0,
			'rows'	=> array()
		);
	}

	$where_str = '';
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false)) {
		$where_str .= "`tp_id` = ? and ";
		array_push($values, $optional['tp_id']);
	}

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`name` like ? or `domain` like ? or `host` like ? or `target_host` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$order_map = array(
		"tp_id",
		"name",
		"domain",
		"type",
		"port",
		"host",
		"target_port",
		"target_host",
		"update_time",
		"create_time",
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}
	
	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `target_host` asc';

	$sql = "select count(*) as `total` from cloud_platform.tunnel_proxy $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		return array(
			'total' => 0,
			'rows'	=> array()
		);
	}

	$sql = "select * from cloud_platform.tunnel_proxy $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function add($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$optional = array();
	if(format_and_push($data, 'name', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}
/*
	if(format_and_push($data, 'port', $optional, '', 'int', false) == false
	|| $optional['port'] <= 0 || $optional['port'] > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '监听端口不正确，必须为1~65535的值。');
		return false;
	}
*/

	if(format_and_push($data, 'target_host', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标主机不能为空。');
		return false;
	}

	if(format_and_push($data, 'target_port', $optional, '', 'int', false) == false
	|| $optional['target_port'] <= 0 || $optional['target_port'] > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标端口不正确，必须为1~65535的值。');
		return false;
	}

	format_and_push($data, 'type', $optional, '', 'int', false);
	format_and_push($data, 'domain', $optional, '', 'string', false);
	format_and_push($data, 'host', $optional, '', 'string', false);
	format_and_push($data, 'path', $optional, '', 'string', false);

	$optional['update_time'] = time();
	$optional['create_time'] = $optional['update_time'];
	try {
		if(insert_data('cloud_platform`.`tunnel_proxy', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$optional['tp_id'] = $nidb->lastInsertId();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	cloud_insertlog($user->username, "创建隧道：主机={$optional['name']}，地址={$optional['target_host']}:{$optional['target_port']}");

	return $optional;
}

function save($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$where = array();
	$optional = array();
	if(format_and_push($data, 'tp_id', $where, '', 'int', false) == false
	|| $where['tp_id'] <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定要修改的隧道。');
		return false;
	}

	if(format_and_push($data, 'name', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}
/*
	if(format_and_push($data, 'port', $optional, '', 'int', false) == false
	|| $optional['port'] <= 0 || $optional['port'] > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '监听端口不正确，必须为1~65535的值。');
		return false;
	}
*/
	if(format_and_push($data, 'target_port', $optional, '', 'int', false) == false
	|| $optional['target_port'] <= 0 || $optional['target_port'] > 65535) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '目标端口不正确，必须为1~65535的值。');
		return false;
	}

	format_and_push($data, 'type', $optional, '', 'int', false);
	format_and_push($data, 'domain', $optional, '', 'string', false);
	format_and_push($data, 'host', $optional, '', 'string', false);
	format_and_push($data, 'target_host', $optional, '', 'string', false);
	format_and_push($data, 'path', $optional, '', 'string', true);

	$optional['update_time'] = time();

	try {
		var_dump( $optional);
		if(update_data('cloud_platform`.`tunnel_proxy', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	cloud_insertlog($user->username, "修改隧道：主机={$optional['name']}，地址={$optional['target_host']}:{$optional['target_port']}");

	return true;
}

function remove($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$optional = array();
	$idsarr = array();
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false) == false 
	|| $optional['tp_id'] <= 0) {
        if(preg_match("/^([0-9,]{1,})$/", $data['ids'], $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "要删除的隧道不正确。");
			return false;
        }
        $optional = explode(',', $data['ids']);
        foreach($optional as $id) {
			$id = intval($id);
			if($id > 0)
				array_push($idsarr, $id);
        }
	}
	else
		 $idsarr = array($optional['tp_id']);

	if(count($idsarr) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '要删除的隧道为空。');
		return false;
	}

	$str = "?";
	if(count($idsarr) > 1)
		$str.= str_repeat(',?', count($idsarr) - 1);

	try {
		
		$sql = "select `tp_id`, `name`, `target_host`, `target_port` from cloud_platform.tunnel_proxy where `tp_id` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($idsarr);
		$rows = $sth->fetchAll(\PDO::FETCH_ASSOC);
		
		$sql = "delete from cloud_platform.tunnel_proxy where `tp_id` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($idsarr);
		
		$sql = "delete from cloud_platform.tunnel_user where `tp_id` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($idsarr);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	foreach($rows as $row) {
		cloud_insertlog($user->username, "删除隧道：主机={$row['name']}，地址={$row['target_host']}:{$row['target_port']}");
	}

	return true;
}


