<?php
namespace cloud\apps\tunnel\sshproxy;


function listen_port_freebsd()
{
	$ports = array();

	// n=端口号转成数字 a=显示所有接口 4=ipv4 L=listen
	$cmd = "netstat -anL4";
	exec($cmd, $out, $ret);
	if($ret == 0) {
/*
Current listen queue sizes (qlen/incqlen/maxqlen)
Proto Listen                           Local Address
tcp4  0/0/128                          *.60443
tcp4  0/0/128                          *.24569
tcp4  0/0/128                          127.0.0.1.9000
tcp46 0/0/80                           *.3306
tcp4  0/0/128                          *.443
tcp4  0/0/128                          *.22
*/
		foreach($out as $line) {
			$line = preg_split('/ +/', $line);
			if(count($line) != 3 
			|| $line[0] != 'tcp4'
			|| $line[2][0] != '*')
				continue;
			$port = intval(substr($line[2], 2));
			array_push($ports, $port);
		}
	}

	return $ports;
}

function listen_port_linux()
{
	$ports = array();

	// n=端口号转成数字 t=tcp 4=ipv4 l=listen
	$cmd = "netstat -nlt4";
	exec($cmd, $out, $ret);
	if($ret == 0) {
/*
Active Internet connections (only servers)
Proto Recv-Q Send-Q Local Address           Foreign Address         State
tcp        0      0 0.0.0.0:50022           0.0.0.0:*               LISTEN
tcp        0      0 127.0.0.1:9000          0.0.0.0:*               LISTEN
tcp        0      0 0.0.0.0:22              0.0.0.0:*               LISTEN
tcp        0      0 0.0.0.0:8791            0.0.0.0:*               LISTEN
tcp        0      0 0.0.0.0:60443           0.0.0.0:*               LISTEN
tcp        0      0 0.0.0.0:443             0.0.0.0:*               LISTEN
*/
		foreach($out as $line) {
			$line = preg_split('/ +/', $line);
			if(count($line) != 6
			|| $line[0] != 'tcp'
			|| substr($line[2], 0, 7) != '0.0.0.0')
				continue;
			$port = intval(substr($line[2], 8));
			array_push($ports, $port);
		}
	}

	return $ports;
}

function gensshkey()
{
	$id_rsa = LOGDATA_DIR . "/tunnel";
	if(!is_dir($id_rsa))
		mkdir($id_rsa);

	$id_rsa.= "/id_rsa";

	if(!file_exists($id_rsa)
	|| !file_exists("/root/.ssh/authorized_keys")) {
		if(file_exists($id_rsa))
			unlink($id_rsa);
		if(file_exists("{$id_rsa}.pub"))
			unlink("{$id_rsa}.pub");
		$cmd = "ssh-keygen -t rsa -P '' -f {$id_rsa} 2>&1";
		exec($cmd, $out, $ret);
		if($ret) return '';

		if(file_exists("/root/.ssh/authorized_keys"))
			unlink("/root/.ssh/authorized_keys");
		if(!rename("{$id_rsa}.pub", "/root/.ssh/authorized_keys"))
			return '';
	}

	return $id_rsa;
}

function get_config()
{
	$config = LOGDATA_DIR . "/tunnel/config.conf";

	if(file_exists($config))
		$ret = myconf_read($config, array('port_min', 'port_max'));
	else
		$ret = array();

	if(isset($ret['port_min']) == false 
	|| isset($ret['port_min']) == false) {
		$ret['port_min'] = 50000;
		$ret['port_max'] = 60000;
	}
	else {
		$ret['port_min'] = intval($ret['port_min']);
		$ret['port_max'] = intval($ret['port_max']);
	}

	return $ret;
}

function get_usableport()
{
	$ret = get_config();
	$port = $ret['port_min'];
	$max_port = $ret['port_max'];

	$ports = dump_sshproxy();

	if(count($ports)) {
		sort($ports, SORT_NUMERIC);
		foreach($ports as $next) {
			if($port != $next)
				break;

			$port++;
			if($port > $max_port)
				return 0;
		}
	}
	
	return $port;
}

function open_sshproxy($target_host, $target_port)
{
	$port = get_usableport();
	if($port == 0) // to max
		return 0;

	if(($id_rsa = gensshkey()) == '')
		return $port;
	
	$sshport = get_sshport();
	if($sshport <= 0)
		return 0;

	$cmd = "ssh -p {$sshport} -o ConnectTimeout=5 -f -N -R :{$port}:{$target_host}:{$target_port} root@127.0.0.1 -i {$id_rsa} > /dev/null 2>&1";
	exec($cmd, $out, $ret);
	if($ret)
		return 0;

	return $port;
}

function dump_sshproxy()
{
	$ports = array();

	$cmd = "ps -Aopid,command";
	exec($cmd, $out, $ret);
	if($ret)
		return $ports;
/*
80809 ssh -p 22 -o ConnectTimeout=5 -f -N -R :60443:112.124.51.144:443 root@192.168.128.220
*/
	$count = 0;
	foreach($out as $line) {
		if (preg_match("/^ *(\d+) .*ssh .*:(\d+):(\d+.\d+.\d+.\d|[^: ]+):(\d+) root@/", $line, $matches)) {
			$port = $matches[2];
			array_push($ports, $port);
		}
	}

	return $ports;
}

function get_sshproxy_pid($port)
{
	$cmd = "ps -Aopid,command";
	exec($cmd, $out, $ret);
	if($ret)
		return 0;

	foreach($out as $line) {
		if (preg_match("/^ *(\d+) .*ssh .*:{$port}:(\d+.\d+.\d+.\d|[^: ]+):(\d+) root@/", $line, $matches)) {
			return $matches[1];
		}
	}

	return 0;
}

function close_sshproxy($port)
{
	$cmd = "ps -Aopid,command";
	exec($cmd, $out, $ret);
	if($ret)
		return 0;
/*
80809 ssh -p 22 -o ConnectTimeout=5 -f -N -R :60443:112.124.51.144:443 root@127.0.0.1
*/
	$count = 0;
	foreach($out as $line) {
		if (preg_match("/^ *(\d+) .*ssh .*:{$port}:(\d+.\d+.\d+.\d|[^: ]+):(\d+) root@/", $line, $matches)) {
			$pid = $matches[1];
			exec("kill {$pid} > /dev/null 2>&1", $err, $ret);
			unset($err);
			$count++;
		}
	}
	
	return $count;
}


