<?php
namespace cloud\apps\tunnel\user;


function select($data)
{
	global $nidb, $user;


	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false)) {
		$where_str .= "`tp_id` = ? and ";
		array_push($values, $optional['tp_id']);
	}
	
	if(!is_supadmin($user->username)) {
		$where_str .= "`user` = ? and ";
		array_push($values, $user->username);
	}
	else {
		if(format_and_push($data, 'user', $optional, '', 'string', false)) {
			$where_str .= "`user` = ? and ";
			array_push($values, $optional['user']);
		}
	}

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`user` like ? or `token` like ? or `last_ip` like ? or `name` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$order_map = array(
		"tp_id",
		"user",
		"name",
		"token",
		"expire_time",
		"count",
		"last_time",
		"last_ip",
		"create_time"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}
	
	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = 'order by `expire_time` asc';

	$sql = "select count(*) as `total` from cloud_platform.tunnel_user $where_str";

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		return array(
			'total' => 0,
			'rows'	=> array()
		);
	}

	$sql = "select * from cloud_platform.tunnel_user $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function add($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$optional = array();
	if(format_and_push($data, 'tp_id', $optional, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定隧道。');
		return false;
	}

	if(format_and_push($data, 'user', $optional, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '帐号不能为空。');
		return false;
	}

	$optional['expire_time'] = 0;
	if(isset($data['expired'])) {
		if($data['expired']) {
			if(format_and_push($data, 'expire_time', $optional, '', 'string', false) == false) {
				$optional['expire_time'] = 0;
			}
			else {
				$expire_time = new \DateTime($optional['expire_time']);
				$optional['expire_time'] = $expire_time->getTimestamp();
			}
		}
	}

	$sql = "select `name` from cloud_platform.tunnel_proxy where `tp_id` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
		$sth->execute();
		$proxy = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$proxy) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "指定的隧道不存在！");
		return false;
	}
	
	$sql = "select `username` from `cloud_users` where `username` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['user'], \PDO::PARAM_STR);
		$sth->execute();
		$cuinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$cuinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "“{$optional['user']}”帐号不存在！");
		return false;
	}
	
	$sql = "select `tp_id` from cloud_platform.tunnel_user where `tp_id` = ? and `user` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['tp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $optional['user'], \PDO::PARAM_STR);
		$sth->execute();
		$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "“{$optional['user']}”帐号的隧道资源“{$proxy['name']}”已存在！");
		return false;
	}

	$now = time();
	$optional['name'] = $proxy['name'];
	$optional['token'] = md5($optional['user'] . $optional['tp_id'] . $now);
	$optional['create_time'] = time();

	try {
		if(insert_data('cloud_platform`.`tunnel_user', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	summary_user($optional['tp_id']);
	
	cloud_insertlog($user->username, "添加用户访问隧道：帐号={$optional['user']}，主机={$proxy['name']}");

	return $optional;
}

function save($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$where = array();
	$optional = array();
	if(format_and_push($data, 'tp_id', $where, '', 'int', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定隧道。');
		return false;
	}

	if(format_and_push($data, 'user', $where, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '帐号不能为空。');
		return false;
	}

	$optional['expire_time'] = 0;
	if(isset($data['expired'])) {
		if($data['expired']) {
			if(format_and_push($data, 'expire_time', $optional, '', 'string', false) == false) {
				$optional['expire_time'] = 0;
			}
			else {
				$expire_time = new \DateTime($optional['expire_time']);
				$optional['expire_time'] = $expire_time->getTimestamp();
			}
		}
	}

	$sql = "select `name` from cloud_platform.tunnel_proxy where `tp_id` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['tp_id'], \PDO::PARAM_INT);
		$sth->execute();
		$proxy = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$proxy) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "指定的隧道不存在！");
		return false;
	}

	$sql = "select `username` from `cloud_users` where `username` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['user'], \PDO::PARAM_STR);
		$sth->execute();
		$cuinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$cuinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "“{$where['user']}”帐号不存在！");
		return false;
	}


	$sql = "select `tp_id` from cloud_platform.tunnel_user where `tp_id` = ? and `user` = ?";

	try {
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['tp_id'], \PDO::PARAM_INT);
		$sth->bindParam(2, $where['user'], \PDO::PARAM_STR);
		$sth->execute();
		$uinfo = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$uinfo) {
		set_errmsg(MSG_LEVEL_EXP, __function__, "“{$where['user']}”帐号不存在！");
		return false;
	}

	if(isset($data['reset']) && intval($data['reset']))
		$optional['token'] = md5($where['user'] . $where['tp_id'] . time());

	$optional['name'] = $proxy['name'];

	try {
		if(update_data('cloud_platform`.`tunnel_user', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	cloud_insertlog($user->username, "修改用户访问隧道：帐号={$where['user']}，主机={$proxy['name']}");

	return $optional;
}

function remove($data)
{
	global $nidb, $user;


	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$idsarr = array();
	$usersarr = array();
	if(isset($data['ids'])) {
		if(preg_match("/^([0-9]+-[^\s]+[,]*)+$/", $data['ids'], $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "要删除的隧道不正确。");
			return false;
        }
        $optional = explode(',', $data['ids']);
        foreach($optional as $idstr) {
			$pos = strpos($idstr, '-');
			$id = intval(substr($idstr, 0, $pos));
			$user = substr($idstr, $pos + 1);
			if($id > 0 && $user != '') {
				array_push($idsarr, $id);
				array_push($usersarr, $user);
			}
        }
		
		if(count($idsarr) <= 0) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请求为空。');
			return false;
		}
	}
	else {
		$optional = array();
		if(format_and_push($data, 'tp_id', $optional, '', 'int', false) == false
		|| $optional['tp_id'] <= 0) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定隧道。');
			return false;
		}
		if(format_and_push($data, 'user', $optional, '', 'string', false) == false
		|| $optional['user'] == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请指定用户。');
			return false;
		}
		array_push($idsarr, $optional['tp_id']);
		array_push($usersarr, $optional['user']);
	}

	
	try {
		$proxys = array();
		$ids = array();
		foreach($idsarr as $key => $val) {
			$sql = "delete from cloud_platform.tunnel_user where `tp_id` = ? and `user` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $idsarr[$key], \PDO::PARAM_INT);
			$sth->bindParam(2, $usersarr[$key], \PDO::PARAM_STR);
			$sth->execute();
			
			if(($pos = array_search($idsarr[$key], $ids)) !== false) {
				$proxy = $proxys[$pos];
			}
			else {
				array_push($ids, $idsarr[$key]);
				
				$sql = "select `name` from cloud_platform.tunnel_proxy where `tp_id` = ?";
				try {
					$sth = $nidb->prepare($sql);
					$sth->bindParam(1, $idsarr[$key], \PDO::PARAM_INT);
					$sth->execute();
					$proxy = $sth->fetch(\PDO::FETCH_ASSOC);
				}
				catch (\NiDBException $e) {
					set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
					continue;
				}
				
				if($proxy)
					array_push($proxys, $proxy);
				else
					array_push($proxys, 0);
			}

			if($proxy)
				cloud_insertlog($user->username, "删除用户访问隧道：帐号={$optional['user']}，主机={$proxy['name']}");
			else
				cloud_insertlog($user->username, "删除用户访问隧道：帐号={$optional['user']}，主机ID=tp_id:{$idsarr[$key]}");
				
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	foreach($ids as $tp_id)
		summary_user($tp_id);

	return true;
}

function summary_user($tp_id)
{
	global $nidb;

	try {
		// 统计计数器
		$sql = "select count(*) as `count`";
		$sql.= " from `cloud_platform`.`tunnel_user`";
		$sql.= " where `tp_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $tp_id, \PDO::PARAM_INT);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);

		if($row) {
			// 更新计数器
			$sql = "update `cloud_platform`.`tunnel_proxy`";
			$sql.= " set `usercnt` = ?";
			$sql.= " where `tp_id` = ?";
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $row['count'], \PDO::PARAM_INT);
			$sth->bindParam(2, $tp_id, \PDO::PARAM_INT);
			$sth->execute();
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

