<?php
namespace cloud\apps\upgrade\option;


function option($data)
{
	global $user;


	if(\cloud\apps\work\project\project_enable(array())) {
		if(!\cloud\apps\work\project\in_work_project(ROLE_GUEST)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}

	$result = array(
		'maxdown'	=> 5,
		'downport'	=> 0,
		'httpshost' => '',
		'httpsport'	=> '',
		'sshdport'	=> '',
	);

	if (file_exists(CLOUD_CONF)) {
		$lines = file(CLOUD_CONF);
		foreach($lines as $val) {
			if (strpos($val, "download_max=") !== false) {
				$ds = explode('=', $val);
				$result['maxdown'] = intval(trim($ds[1], "\n"));
			}
			if (strpos($val, "httpshost=") !== false) {
				$ds = explode('=', $val);
				$result['httpshost'] = trim($ds[1], "\n");
			}
			else
			if (strpos($val, "httpsport=") !== false) {
				$ds = explode('=', $val);
				$result['httpsport'] = intval(trim($ds[1], "\n"));
			}
			else
			if (strpos($val, "sshdport=") !== false) {
				$ds = explode('=', $val);
				$result['sshdport'] = intval(trim($ds[1], "\n"));
			}
		}
	}

	$nginxconf_cloud = "/usr/local/etc/nginx/nginx.conf.cloud";
	$fp = fopen($nginxconf_cloud, "r");
	if ($fp) {
		$httpport = 0;
		while (!feof($fp)) {
			$buff = fgets($fp);
			if ($buff !== false) {
				if (strstr($buff, "listen") !== false && 
					strstr($buff, "ssl") === false && 
					strstr($buff, "#") === false) {
						for ($i = 0; $i < strlen($buff); $i++) {
							$poschr = substr($buff, $i, 1);
							if ($poschr >= '0' && $poschr <= '9')
								$httpport = $httpport * 10 + ($poschr - '0');
						}
					}
			}
		}
		fclose($fp);
		$result['downport'] = $httpport;
	}

	return $result;
}

function save($data)
{
	global $user;


	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}

	if(isset($data['maxdown'])) {
		$maxdown = intval($data['maxdown']);
		if($maxdown <= 0) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '同时升级最大设备数不正确！');
			return false;
		}
	}
	if(isset($data['downport'])) {
		$downport = intval($data['downport']);
		if($downport <= 0 || $downport > 65535) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '文件下载端口不正确！');
			return false;
		}
	}
	if(isset($data['httpshost'])) {
		$httpshost = trim($data['httpshost']);
	}
	if(isset($data['httpsport'])) {
		$httpsport = intval($data['httpsport']);
		if($httpsport < 0 || $httpsport > 65535) {
			set_errmsg(MSG_LEVEL_EXP, __function__, 'HTTPS外网映射端口不正确！');
			return false;
		}
	}
	if(isset($data['sshdport'])) {
		$sshdport = intval(trim($data['sshdport']));
		if($sshdport < 0 || $sshdport > 65535) {
			set_errmsg(MSG_LEVEL_EXP, __function__, 'SSHD外网映射端口不正确！');
			return false;
		}
	}

	if(isset($maxdown) 
	|| (isset($httpsport) && isset($httpshost))
	|| isset($sshdport)) {

		if(isset($maxdown))
			cloud_insertlog($user->username, "配置同时升级最大设备数改为{$maxdown}");	
		if(isset($httpsport) && isset($httpshost)) {
			if($httpshost != "" && $httpsport > 0)
				cloud_insertlog($user->username, "配置HTTPS外网映射改为https://{$httpshost}:{$httpsport}");	
		}
		if(isset($sshdport)) 
			cloud_insertlog($user->username, "配置SSHD外网映射端口改为{$sshdport}");

		if (!file_exists(CLOUD_CONF)) {
			if (($fp = fopen(CLOUD_CONF, "w")) === false) {
				set_errmsg(MSG_LEVEL_EXP, __function__, '配置文件不存在！');
				return false;
			}
			if(isset($maxdown))
				fwrite($fp, "download_max={$maxdown}\n");
			if(isset($httpshost) && $httpshost != "")
				fwrite($fp, "httpshost={$httpshost}\n");
			if(isset($httpsport) && $httpsport > 0)
				fwrite($fp, "httpsport={$httpsport}\n");
			if(isset($sshdport) && $sshdport > 0)
				fwrite($fp, "sshdport={$sshdport}\n");
			fclose($fp);
		}
		else {
			$lines = file(CLOUD_CONF);
			if (($fp = fopen(CLOUD_CONF . '.cloud.web', "w")) != false) {
				$fd = 0;
				foreach($lines as $val) {
					if (strpos($val, "download_max=") !== false && isset($maxdown)) {
						fwrite($fp, "download_max={$maxdown}\n");
						$fd |= 1;
						continue;
					}
					if (strpos($val, "httpshost=") !== false && isset($httpshost)) {
						if($httpshost != "")
							fwrite($fp, "httpshost={$httpshost}\n");
						$fd |= 2;
						continue;
					}
					if (strpos($val, "httpsport=") !== false && isset($httpsport)) {
						if($httpsport > 0)
							fwrite($fp, "httpsport={$httpsport}\n");
						$fd |= 4;
						continue;
					}
					if (strpos($val, "sshdport=") !== false && isset($sshdport)) {
						if($sshdport > 0)
							fwrite($fp, "sshdport={$sshdport}\n");
						$fd |= 8;
						continue;
					}
					fwrite($fp, $val);
				}
				if (($fd & 1)== 0 && isset($maxdown))
					fwrite($fp, "download_max={$maxdown}\n");
				if (($fd & 2)== 0 && isset($httpshost)) {
					if($httpshost != "")
						fwrite($fp, "httpshost={$httpshost}\n");
				}
				if (($fd & 4)== 0 && isset($httpsport)) {
					if($httpsport > 0)
						fwrite($fp, "httpsport={$httpsport}\n");
				}
				if (($fd & 8)== 0 && isset($sshdport)) {
					if($sshdport > 0)
						fwrite($fp, "sshdport={$sshdport}\n");
				}
				fclose($fp);
				unlink(CLOUD_CONF);
				rename(CLOUD_CONF . '.cloud.web', CLOUD_CONF);
			}
		}
	}

	if(isset($downport)) {

		cloud_insertlog($user->username, "文件下载端口修改为{$downport}");	

		$nginxconf_cluster  = "/usr/local/etc/nginx/nginx.conf.cluster";
		$nginxconf_cloud = "/usr/local/etc/nginx/nginx.conf.cloud";

		if(PHP_OS == "Linux" 
		&& is_link($nginxconf_cloud)) {
			$nginxconf_cloud = "/usr/logd/nginx/conf/vhost/download.conf";

			if($downport > 0) {
				if (!file_exists($nginxconf_cloud)) {
$download = "
	server {
		listen  {$downport};
		server_name  localhost;

		#charset koi8-r;

		#access_log  logs/host.access.log  main;

		location / {
				root /usr/logd/www/;
			index  index.php index.html index.htm;

			fastcgi_connect_timeout 3600;
			fastcgi_read_timeout 3600;
			fastcgi_send_timeout 3600;
		}

		#error_page  404              /404.html;

		# redirect server error pages to the static page /50x.html
		#
		error_page   500 502 503 504  /50x.html;
		location = /50x.html {
				#root /usr/logd/www/;
		}

		# proxy the PHP scripts to Apache listening on 127.0.0.1:80
		#
		#location ~ \.php$ {
		#    proxy_pass   http://127.0.0.1;
		#}

		# pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
		#
		location ~ \.php$ {
				#root /usr/logd/www/;
			fastcgi_pass   127.0.0.1:9000;

			fastcgi_connect_timeout 600;
			fastcgi_read_timeout 3600;
			fastcgi_send_timeout 3600;

			fastcgi_index  index.php;
			#fastcgi_param  SCRIPT_FILENAME  /scripts$fastcgi_script_name;
				#root /usr/logd/www/;
			include        fastcgi_params;
			client_max_body_size    40m;
		}

		# deny access to .htaccess files, if Apache's document root
		# concurs with nginx's one
		#
		#location ~ /\.ht {
		#    deny  all;
		#}
	}
";
					file_put_contents($nginxconf_cloud, $download);
				}
				else {
						
					$lines = file($nginxconf_cloud);

					$s = 0;	
					foreach($lines as $k => $val) {
						if (strstr($val, "listen") !== false && strstr($val, "ssl") === false && strstr($val, "#") === false)
							$lines[$k] = "	listen	{$downport};\n";

						if (strstr($val, "/usr/logd/www") != false && $s == 0) {
							$s = 1;
							continue;
						}

						if (strstr($val, "/usr/logd/www") != false && $s == 1) {
							$s = 1;
							$lines[$k] = "		#root /usr/logd/www/;\n";
							continue;
						}

						if (strstr($val, "/usr/logdata/file_cluster") != false && $s == 0) {
							$s = 1;
							$lines[$k] = "		root /usr/logd/www/;\n";
						}
					}

					if (($fp = fopen($nginxconf_cloud, "w")) != false) {
						foreach($lines as $k => $val) 
							fwrite($fp, $val);

						fclose($fp);
					}
				}
			}
			else
				unlink($nginxconf_cloud);
			
			system("killall nginx && /usr/local/sbin/nginx");
		}
		else {
			
			if (!file_exists($nginxconf_cloud))
				copy($nginxconf_cluster, $nginxconf_cloud);

			$lines = file($nginxconf_cloud);

			$s = 0;	
			foreach($lines as $k => $val) {
				if (strstr($val, "listen") !== false && strstr($val, "ssl") === false && strstr($val, "#") === false)
					$lines[$k] = "	listen	{$downport};\n";

				if (strstr($val, "/usr/logd/www") != false && $s == 0) {
					$s = 1;
					continue;
				}

				if (strstr($val, "/usr/logd/www") != false && $s == 1) {
					$s = 1;
					$lines[$k] = "		#root /usr/logd/www/;\n";
					continue;
				}

				if (strstr($val, "/usr/logdata/file_cluster") != false && $s == 0) {
					$s = 1;
					$lines[$k] = "		root /usr/logd/www/;\n";
				}
			}

			if (($fp = fopen($nginxconf_cloud, "w")) != false) {
				foreach($lines as $k => $val) 
					fwrite($fp, $val);

				fclose($fp);
			}

			exec('ps axwww | grep \'nginx.conf.cloud\' | grep -v grep | awk \'{print $1}\'', $out, $ret);
			foreach($out as $val) {
				$pid = trim($val, "\n");
				system("/bin/logeye killpro {$pid} >/dev/null 2>&1");
			}

			if ($downport != 0) {
				$nginx = '/usr/local/sbin/nginx';
				if (file_exists($nginx))
					system("{$nginx} -c {$nginxconf_cloud}");
				$nginx = '/usr/sbin/nginx';
				if (file_exists($nginx))
					system("{$nginx} -c {$nginxconf_cloud}");
			}
		}
	}

	if(isset($downport)
	|| (isset($httpsport) && isset($httpshost))
	|| isset($sshdport)) {

		if(file_exists(PANABIT_BAGS_PATH . "/webport.conf"))
			$lines = file(PANABIT_BAGS_PATH . "/webport.conf");
		else
			$lines = array();
		if (($fp = fopen(PANABIT_BAGS_PATH . "/webport.conf.cloud.web", "w")) != false) {
			$fd = 0;
			foreach($lines as $val) {
				if (strpos($val, "WEBPORT=") !== false && isset($downport)) {
					fwrite($fp, "WEBPORT={$downport}\n");
					$fd |= 1;
					continue;
				}
				if (strpos($val, "SSHDPORT=") !== false && isset($sshdport)) {
					if($sshdport > 0)
						fwrite($fp, "SSHDPORT={$sshdport}\n");
					$fd |= 2;
					continue;
				}
				if (strpos($val, "DEVOPS_API_URL=") !== false && (isset($httpshost) && isset($httpsport))) {
					if($httpshost != '' && $httpsport > 0)
						fwrite($fp, "DEVOPS_API_URL=https://{$httpshost}:{$httpsport}/cloud/task.php\n");
					$fd |= 4;
					continue;
				}
				fwrite($fp, $val);
			}
			if (($fd & 1)== 0 && isset($downport)) {
				fwrite($fp, "WEBPORT={$downport}\n");
			}
			if (($fd & 2)== 0 && isset($sshdport)) {
				if($sshdport > 0)
					fwrite($fp, "SSHDPORT={$sshdport}\n");
			}
			if (($fd & 4)== 0 && (isset($httpshost) && isset($httpsport))) {
				if($httpshost != '' && $httpsport > 0)
					fwrite($fp, "DEVOPS_API_URL=https://{$httpshost}:{$httpsport}/cloud/task.php\n");
			}
			fclose($fp);
			if(file_exists(PANABIT_BAGS_PATH . "/webport.conf"))
				unlink(PANABIT_BAGS_PATH . "/webport.conf");
			rename(PANABIT_BAGS_PATH . "/webport.conf.cloud.web", PANABIT_BAGS_PATH . "/webport.conf");
		}
	}
	
	return true;
}