<?php
namespace cloud\apps\user;


function select($data)
{
	global $nidb, $user;


	if(\cloud\apps\work\project\project_enable(array())) {
		if(!\cloud\apps\work\project\in_work_project(ROLE_GUEST)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = '';

	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`username` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	$order_map = array(
		"username",
		"logintime",
		"grp",
		"grpname",
		"remarks",
		"loginip",
		"ixc_grpids",
		"ixc_grpnames",
		"log_grpids",
		"log_grpnames",
		"intool_grpids",
		"intool_grpnames"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($data['columns'][$op['column']]['data'])) {
				if(in_array($data['columns'][$op['column']]['data'], $order_map))
					array_push($order, "`" . $data['columns'][$op['column']]['data'] . "` " . $op['dir']);
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_users $where_str";
	try {
		$sth = $nidb->prepare($sql);

		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result = array(
			'total' => $rows[0]->total,
		);
	}
	else {
		$result = array(
			'total' => 0,
		);
	}

	$keys = '`username`,
`mobile`,
`logintime`,
`grp`,
`grpname`,
`httpacct`,
`remarks`,
`loginip`,
`ixc_grpids`,
`ixc_grpnames`,
`log_grpids`,
`log_grpnames`,
`intool_grpids`,
`intool_grpnames`';

	$sql = "select $keys from cloud_users $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	$result['rows'] = $rows;

	
	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function setsupadmin($username, $data)
{
	global $nidb, $user;


	if($user->username == ADMIN_ACCT
	&& $user->username != $username
	&& (isset($data['forever']) || isset($data['expire']))) {
		$dir = LOGDATA_DIR . "/supadmin";
		if(!is_dir($dir)) {
			if (!mkdir($dir)) {
				set_errmsg(MSG_LEVEL_ARG, __function__, "创建目录失败！{$dir}");
				return false;
			}
		}

		$userfile = "{$dir}/{$username}";
		if(isset($data['forever'])) {
			$forever = intval($data['forever']);
			if($forever == 0) {
				if(file_exists($userfile))
					unlink($userfile);
				
				cloud_insertlog($user->username, "关闭超管权限：{$username}");	
				return true;
			}
		}

		if(file_exists($userfile)) {
			$ring = file_get_contents($userfile);
			$ring = json_decode($ring, true);
		}
		if(!$ring) {
			$ring = array(
				'forever' => 0,
				'expire' => 0,
			);
		}
		
		$strdesc = array();
		if(isset($data['forever'])) {
			$ring['forever'] = $forever;
			if($forever == 1) {
				array_push($strdesc, '永久');
			}
		}

		if(isset($data['expire'])) {
			// expire=2016-12-14
			// or
			// expire=2016-12-14 23:59:59
			if(preg_match("/^\d{4}[\-](0?[1-9]|1[012])[\-](0?[1-9]|[12][0-9]|3[01])$/", $data['expire'])
			|| preg_match("/^\d{4}[\-](0?[1-9]|1[012])[\-](0?[1-9]|[12][0-9]|3[01])(\s+(0?[0-9]|1[0-9]|2[0-3])\:(0?[0-9]|[1-5][0-9])\:(0?[0-9]|[1-5][0-9]))?$/", $data['expire'])) {
				$expire = new \DateTime($data['expire']);
				$ring['expire'] = $expire->getTimestamp();
				if($ring['forever'] == 2)
					array_push($strdesc, '有效截止时间=' . date('Y-m-d H:i:s', $ring['expire']));
			}
		}

		if(!file_put_contents($userfile, json_encode($ring))) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '保存配置失败！');
			return false;
		}
		cloud_insertlog($user->username, "设置超管权限：{$username} " . implode(',', $strdesc));
	}

	return true;
}

function supadmin($data)
{
	global $user;


	$optional = array();
	if(format_and_push($data, 'user', $optional, 'username', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '用户名不能为空。');
		return false;
	}
	$optional['username'] = strtolower($optional['username']);

	if(!is_supadmin($user->username)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}
	
	if(isset($data['forever']) || isset($data['expire']))
		return setsupadmin($optional['username'], $data);
	
	$userfile = LOGDATA_DIR . "/supadmin/{$optional['username']}";
	if(file_exists($userfile)) {
		$ring = file_get_contents($userfile);
		$ret = json_decode($ring, true);
		if($ret) {
			if(time() > $ret['expire'])
				$ret['expired'] = 1;
			else
				$ret['expired'] = 0;

			return $ret;
		}
	}

	return array(
		'forever' => 0,
		'expire' => 0,
		'expired' => 1,
	);
}

function add($data, $allow = 0)
{
	global $nidb, $user;


	$optional = array();
	if(format_and_push($data, 'user', $optional, 'username', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '用户名不能为空。');
		return false;
	}

	$optional['username'] = strtolower($optional['username']);

	if(!$allow) {
		if(\cloud\apps\work\project\project_enable(array())) {
			if(!\cloud\apps\work\project\in_work_user($optional['username'])) {
				set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
				return false;
			}
		}
		else
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
			return false;
		}
	}

	if(format_and_push($data, 'pass', $optional, 'password', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '用户密码不能为空。');
		return false;
	}

	if(format_and_push($data, 'remarks', $optional, '', 'string', false) == false)
		$optional['remarks']	= '';

	$optional['ixc_grpids']		= '';
	$optional['ixc_grpnames']	= '';
	$optional['grp']			= '';
	$optional['grpname']		= '';
	$optional['log_grpids']		= '';
	$optional['log_grpnames']	= '';
	$optional['intool_grpids']	= '';
	$optional['intool_grpnames']= '';
	$optional['password']		= md5($optional['password']);

	if(!$allow)
		cloud_insertlog($user->username, "添加用户：{$optional['username']}");	

	try {
		$sql = "select * from cloud_users where `username` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['username'], \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if($row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该用户已存在！');
			return false;
		}

		if(insert_data('cloud_users', $optional) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}

		return true;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// setsupadmin($optional['username'], $data);

	return false;
}

function passwd($data)
{
	global $nidb, $user;
	

	$optional = array();
	$tmp = array();
	if(format_and_push($data, 'user', $optional, 'username', 'string', false) == false)
		$optional['username'] = $user->username;

	if(\cloud\apps\work\project\project_enable(array())) {
		if(!\cloud\apps\work\project\in_work_user($optional['username'])) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}
	else
	if(!is_supadmin($user->username)) {
		if($optional['username'] != $user->username) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
			return false;
		}
	}

	if(format_and_push($data, 'oldpass', $tmp, '', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '原密码不能为空。');
		return false;
	}
	
	if(format_and_push($data, 'pass', $optional, 'password', 'string', false) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '修改的密码不能为空。');
		return false;
	}

	try {
		$sql = "select * from cloud_users where `username` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['username'], \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该用户不存在！');
			return false;
		}

		if(!($user->username == ADMIN_ACCT && $optional['username'] != $user->username)) {
			$oldpass = md5($tmp['oldpass']);
			if($oldpass != $row['password']) {
				set_errmsg(MSG_LEVEL_DEF, __function__, '原密码不正确！');
				return false;
			}
		}

		$optional['password'] = md5($optional['password']);
		if(update_data('cloud_users', $optional, array('username' => $optional['username'])) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}

	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($optional['username'] != $user->username)
		cloud_insertlog($user->username, "修改密码{$optional['username']}");
	else
		cloud_insertlog($user->username, "修改密码");

	if($optional['username'] == $user->username)
		$_SESSION['cloud_user_lastpass'] = $optional['password'];

	return true;
}

function mobile($data)
{
	global $nidb, $user;

	$optional = array();
	if(format_and_push($data, 'user', $optional, 'username', 'string', false) == false) {
		if(format_and_push($data, 'username', $optional, '', 'string', false) == false)
			$optional['username'] = $user->username;
	}

	if(\cloud\apps\work\project\project_enable(array())) {
		if(!\cloud\apps\work\project\in_work_project(ROLE_TECHN)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}
	else
	if(!is_supadmin($user->username)) {
		if($optional['username'] != $user->username) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
			return false;
		}
	}

//	format_and_push($data, 'oldpass', $optional, '', 'string', false);
	if(format_and_push($data, 'phone', $optional, 'mobile', 'string', false) == false) {
		$optional['mobile'] = '';
		$tag = '解绑';
	}
	else 
		$tag = '绑定';

	try {
		$sql = "select * from cloud_users where `username` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['username'], \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该用户不存在！');
			return false;
		}
/*
		// 如果设置了密码，则验证密码。
		if(isset($optional['oldpass'])) {
			if(!($user->username == ADMIN_ACCT && $optional['username'] != $user->username)) {
				if($row['password'] != md5($optional['oldpass'])) {
					set_errmsg(MSG_LEVEL_DEF, __function__, '原密码不正确！');
					return false;
				}
			}
		}
*/
		if(update_data('cloud_users', $optional, array('username' => $optional['username'])) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if($optional['username'] != $user->username)
		cloud_insertlog($user->username, $tag . "手机号码={$optional['mobile']}，帐号={$optional['username']}");
	else
		cloud_insertlog($user->username, $tag . "手机号码={$optional['mobile']}");

	return true;
}

function save_httpacct($data)
{
	global $nidb, $user;

	$optional = array();
	if(format_and_push($data, 'user', $optional, 'username', 'string', false) == false)
		$optional['username'] = $user->username;

	if(\cloud\apps\work\project\project_enable(array())) {
		if(!\cloud\apps\work\project\in_work_project(ROLE_TECHN)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}
	else
	if(!is_supadmin($user->username)) {
		if($optional['username'] != $user->username) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
			return false;
		}
	}

	if(!isset($data['httpacct'])) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定网关HTTP帐号。');
		return false;
	}
	$optional['httpacct'] = trim($data['httpacct']);

	try {
		$sql = "select * from cloud_users where `username` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['username'], \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该用户不存在！');
			return false;
		}

		if(update_data('cloud_users', $optional, array('username' => $optional['username'])) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if($optional['username'] != $user->username)
		cloud_insertlog($user->username, $tag . "修改HTTP、HTTPS远程访问帐号：{$optional['username']}.{$optional['httpacct']}");
	else
		cloud_insertlog($user->username, $tag . "修改HTTP、HTTPS远程访问帐号：{$optional['httpacct']}");

	return true;
}

function get_gateway_group($grpids)
{
	global $nidb;

	
	$result = array(
		'id'	=> array(),
		'name'	=> array()
	);

	$str = "?";
	if(count($grpids) > 1)
		$str.= str_repeat(',?', count($grpids) - 1);

	try {
		$sql = "select `grpid` as `id`, `grpname` as `name` from cloud_grp where `grpid` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($grpids);

		while ($row = $sth->fetch(\PDO::FETCH_ASSOC)) {
			array_push($result['id'], $row['id']);
			array_push($result['name'], $row['name']);
		}
		$sth = null;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $result;
}

function get_ixcache_group($grpids)
{
	global $nidb;

	
	$result = array(
		'id'	=> array(),
		'name'	=> array()
	);
	
	$str = "?";
	if(count($grpids) > 1)
		$str.= str_repeat(',?', count($grpids) - 1);

	try {
		$sql = "select `grpid` as `id`, `grpname` as `name` from ixcache_grp where `grpid` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($grpids);

		while ($row = $sth->fetch(\PDO::FETCH_ASSOC)) {
			array_push($result['id'], $row['id']);
			array_push($result['name'], $row['name']);
		}
		$sth = null;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $result;
}

function get_panalog_group($grpids)
{
	global $nidb;

	
	$result = array(
		'id'	=> array(),
		'name'	=> array()
	);
	
	$str = "?";
	if(count($grpids) > 1)
		$str.= str_repeat(',?', count($grpids) - 1);

	try {
		$sql = "select `grpid` as `id`, `grpname` as `name` from logserver_grp where `grpid` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($grpids);

		while ($row = $sth->fetch(\PDO::FETCH_ASSOC)) {
			array_push($result['id'], $row['id']);
			array_push($result['name'], $row['name']);
		}
		$sth = null;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $result;
}

function get_intool_group($grpids)
{
	global $nidb;


	$result = array(
		'id'	=> array(),
		'name'	=> array()
	);
	
	$str = "?";
	if(count($grpids) > 1)
		$str.= str_repeat(',?', count($grpids) - 1);
	
	try {
		$sql = "select `id`, `name` from intool_grp where `id` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($grpids);

		while ($row = $sth->fetch(\PDO::FETCH_ASSOC)) {
			array_push($result['id'], $row['id']);
			array_push($result['name'], $row['name']);
		}
		$sth = null;
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $result;
}

function save($data)
{
	global $nidb, $user;
	

	$optional = array();
	if(format_and_push($data, 'user', $optional, 'username', 'string', false) == false)
		$optional['username'] = $user->username;
	
	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}
	else
	if(!is_supadmin($user->username)) {
		if($optional['username'] != $user->username) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
			return false;
		}
	}
	
	$pagrp = array();
	$ixgrp = array();
	$loggrp = array();
	$ingrp = array();

	$grplist = array('gateway', 'ixcache', 'panalog', 'intranet');

	if(isset($data['grp']) && is_string($data['grp'])) {
		$str = explode('@', $data['grp']);
		foreach($str as $group) {
			$node = explode('|', $group);
			if(count($node) <= 0)
				continue;
			$grpname = $node[0];
			if(in_array($grpname, $grplist) === false)
				continue;
			if(count($node) == 1)
				$groups = array(0);
			else
				$groups = explode(',', $node[1]);

			if($grpname == 'gateway') {
				foreach($groups as $id) {
					if(($grpid = intval($id)) < 0) {
						set_errmsg(MSG_LEVEL_ARG, __function__, $grpname . '更新组的ID不正确。');
						return false;
					}
					array_push($pagrp, $grpid);
				}
			}
			else
			if($grpname == 'ixcache') {
				foreach($groups as $id) {
					if(($grpid = intval($id)) < 0) {
						set_errmsg(MSG_LEVEL_ARG, __function__, $grpname . '更新组的ID不正确。');
						return false;
					}
					array_push($ixgrp, $grpid);
				}
			}
			else
			if($grpname == 'panalog') {
				foreach($groups as $id) {
					if(($grpid = intval($id)) < 0) {
						set_errmsg(MSG_LEVEL_ARG, __function__, $grpname . '更新组的ID不正确。');
						return false;
					}
					array_push($loggrp, $grpid);
				}
			}
			else
			if($grpname == 'intranet') {
				foreach($groups as $id) {
					if(($grpid = intval($id)) < 0) {
						set_errmsg(MSG_LEVEL_ARG, __function__, $grpname . '更新组的ID不正确。');
						return false;
					}
					array_push($ingrp, $grpid);
				}
			}
		}
	}

	$frmData = array();
	if(count($pagrp)) {
		if(in_array(0, $pagrp)) {
			$frmData['grp']			= '';
			$frmData['grpname']		= '';
		}
		else {
			$grp = get_gateway_group($pagrp);
			if($grp && count($grp)) {
				$frmData['grp']		= implode(',', $grp['id']);
				$frmData['grpname'] = iconv("gbk", "utf-8", implode(',', $grp['name']));
				if(strlen($frmData['grpname']) > 512) {
					$str = substr($frmData['grpname'], 0, 508);
					$tmp = explode(',', $str);
					array_pop($tmp);
					$frmData['grpname'] = implode(',', $tmp) . ',...';
				}
			}
		}
	}
	if(count($ixgrp)) {
		if(in_array(0, $ixgrp)) {
			$frmData['ixc_grpids']			= '';
			$frmData['ixc_grpnames']		= '';
		}
		else {
			$grp = get_ixcache_group($ixgrp);
			if($grp && count($grp)) {
				$frmData['ixc_grpids']		= implode(',', $grp['id']);
				$frmData['ixc_grpnames']	= implode(',', $grp['name']);
				if(strlen($frmData['ixc_grpnames']) > 512) {
					$str = substr($frmData['ixc_grpnames'], 0, 508);
					$tmp = explode(',', $str);
					array_pop($tmp);
					$frmData['ixc_grpnames'] = implode(',', $tmp) . ',...';
				}
			}
		}
	}
	if(count($loggrp)) {
		if(in_array(0, $loggrp)) {
			$frmData['log_grpids']			= '';
			$frmData['log_grpnames'] 		= '';
		}
		else {
			$grp = get_panalog_group($loggrp);
			if($grp && count($grp)) {
				$frmData['log_grpids']		= implode(',', $grp['id']);
				$frmData['log_grpnames']	= implode(',', $grp['name']);
				if(strlen($frmData['log_grpnames']) > 512) {
					$str = substr($frmData['log_grpnames'], 0, 508);
					$tmp = explode(',', $str);
					array_pop($tmp);
					$frmData['log_grpnames'] = implode(',', $tmp) . ',...';
				}
			}
		}
	}
	if(count($ingrp)) {
		if(in_array(0, $ingrp)) {
			$frmData['intool_grpids']		= '';
			$frmData['intool_grpnames']		= '';
		}
		else {
			$grp = get_intool_group($ingrp);
			if($grp && count($grp)) {
				$frmData['intool_grpids']	= implode(',', $grp['id']);
				$frmData['intool_grpnames']	= implode(',', $grp['name']);
				if(strlen($frmData['intool_grpnames']) > 512) {
					$str = substr($frmData['intool_grpnames'], 0, 508);
					$tmp = explode(',', $str);
					array_pop($tmp);
					$frmData['intool_grpnames'] = implode(',', $tmp) . ',...';
				}
			}
		}
	}

	if(isset($data['remarks']))
		format_and_push($data, 'remarks', $frmData, '', 'string', true);

	if(count($frmData) == 0) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '请指定要保存的内容！');
		return false;
	}

	cloud_insertlog($user->username, "用户配置修改{$optional['username']}");


	try {
		$sql = "select * from cloud_users where `username` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $optional['username'], \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该用户不存在！');
			return false;
		}

		if(update_data('cloud_users', $frmData, array('username' => $optional['username'])) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	// setsupadmin($optional['username'], $data);

	return true;
}

function remove($data)
{
	global $nidb, $user;
	

	if(\cloud\apps\work\project\project_enable(array())) {
		if(!is_supadmin($user->username)) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足！');
			return false;
		}
	}
	else
	if(!is_supadmin($user->username)) {
		if($optional['username'] != $user->username) {
			set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
			return false;
		}
	}

	if(isset($data['u']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '要删除的用户不能为空。');
		return false;
	}
	if(is_array($data['u']) === false) {
		if(empty($data['u']) || gettype($data['u']) != 'string') {
			set_errmsg(MSG_LEVEL_ARG, __function__, '请选择要删除的用户。');
			return false;
		}

		$users = array($data['u']);
	}
	else
		$users = $data['u'];

	if(count($users) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '要删除的用户为空。');
		return false;
	}

	cloud_insertlog($user->username, "删除用户" . implode(',', $users));	

	$str = "?";
	if(count($users) > 1)
		$str.= str_repeat(',?', count($users) - 1);

	try {
		$sql = "delete from cloud_users where `username` in ({$str})";
		$sth = $nidb->prepare($sql);
		$sth->execute($users);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	return true;
}

function menu_save($data)
{
	global $nidb, $user;


	$where = array();
	$optional = array();
	if(format_and_push($data, 'user', $where, 'username', 'string', false) == false)
		$where['username'] = $user->username;
	
	if($account != ADMIN_ACCT) {
		if(!is_supadmin($user->username)) {
			if($where['username'] != $user->username) {
				set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
				return false;
			}
		}
	}
	
	if(format_and_push($data, 'menus', $optional, '', 'string', false) == false)
		$optional['menus'] = '_EMPTY_';

	cloud_insertlog($user->username, "用户菜单修改：{$where['username']}");	

	try {
		$sql = "select * from cloud_users where `username` = ? limit 1";

		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $where['username'], \PDO::PARAM_STR);
		$sth->execute();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
		if(!$row) {
			set_errmsg(MSG_LEVEL_DEF, __function__, '该用户不存在！');
			return false;
		}

		if(update_data('cloud_users', $optional, $where) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return true;
}

function get_user_menu($account)
{
	global $nidb, $default_nav_menu;


	$menus_count = 0;
	if($account != ADMIN_ACCT) {

		$sql = "select * from cloud_users where `username` = ? limit 1";

		try {
			$sth = $nidb->prepare($sql);
			$sth->bindParam(1, $account, \PDO::PARAM_STR);
			$sth->execute();

			$acct = $sth->fetch(\PDO::FETCH_ASSOC);
		} catch (\PDOException $e) {
			set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
			return false;
		}

		if(!$acct) {
			set_errmsg(MSG_LEVEL_EXP, __function__, '无效的帐号！');
			return false;
		}

		if(isset($acct['menus']) && $acct['menus'] != "") {
			$menus = explode(',', $acct['menus']);
			$menus_count = count($menus);
		}
	}

	$ret = array();
	//  2,3,7,8,9,10,11,12,13,14,15,101,102,103,104,105,106
/*
string(15) " 1 网关设备"
string(16) "1 2 缓存设备"
string(16) "1 3 日志设备"
string(9) " 4 SD-WAN"
string(6) " 5 SAC"
string(12) " 6 Web代理"
string(16) "1 7 设备地图"
string(19) "1 8 自定义拓扑"
string(16) "1 9 设备升级"
string(17) "1 10 系统升级"
string(14) "1 11 防火墙"
string(17) "1 12 用户管理"
string(17) "1 13 系统日志"
string(17) "1 14 一键断网"
string(17) "1 15 微信通知"
*/
// var_dump($acct['menus'], $menus);
	foreach($default_nav_menu as $menu)
	{
		$m = array(
			'id'		=> $menu['id'],		// 唯一ID
			'name'		=> $menu['name'],	// 菜单名称
			'ico'		=> $menu['ico'],	// 菜单图标
			'url'		=> $menu['url'],	// URL链接地址
			'enabled'	=> 1,				// 是否显示此菜单
			'existed'	=> 1,				// 菜单对应的模块是否存在
			'children'	=> array(),			// 二菜子菜单
		);

		if($menus_count 
		&& !in_array($menu['id']."", $menus))
			$m['enabled'] = 0;
// var_dump(in_array($menu['id']."", $menus) . ' ' .  $menu['id'] . ' ' . $menu['name']);
		if (isset($menu['file_exist']) && $menu['file_exist'] != '') {
			if (!file_exists(ROOT_DIR . $menu['file_exist']))
				$m['existed'] = 0;
		}
		if (isset($menu['user']) && $menu['user'] != '') {
			if ($menu['user'] != $account)
				$m['existed'] = 0;
		}
		if (isset($menu['supadmin']) && $menu['supadmin']) {
			if (!is_supadmin($account)) 
				$m['existed'] = 0;
		}

		// 输出子菜单
		if (isset($menu['children'])) {
			// 输出一级菜单
			$m['enabled'] = 0;

			foreach($menu['children'] as $chmenu) 
			{
				$chm = array(
					'id'		=> $chmenu['id'],	// 唯一ID
					'name'		=> $chmenu['name'],	// 菜单名称
					'ico'		=> $chmenu['ico'],	// 菜单图标
					'url'		=> $chmenu['url'],	// URL链接地址
					'enabled'	=> 1,				// 是否显示此菜单
					'existed'	=> 1,				// 菜单对应的模块是否存在
					'children'	=> array(),			// 二菜子菜单
				);
				
				$enabled = 0;
				if($menus_count 
				&& !in_array($chmenu['id']."", $menus))
					$chm['enabled'] = 0;
				else
					$enabled = 1;

				if (isset($chmenu['file_exist']) && $chmenu['file_exist'] != '') {
					if (!file_exists(ROOT_DIR . $chmenu['file_exist']))
						$chm['existed'] = 0;
				}
				if (isset($chmenu['user']) && $chmenu['user'] != '') {
					if ($chmenu['user'] != $account)
						$chm['existed'] = 0;
					else
						$enabled = 1;
				}
				if (isset($chmenu['supadmin']) && $chmenu['supadmin']) {
					if (!is_supadmin($account)) 
						$chm['existed'] = 0;
					else
						$enabled = 1;
				}
				// 输出二级子菜单
				array_push($m['children'], $chm);
				
				if($enabled)
					$m['enabled']++;
			}
		}

		array_push($ret, $m);
	}

	return $ret;
}