<?php
namespace cloud\apps\zeropen\config;

defined('OPSZEROPEN_DIR') or define('OPSZEROPEN_DIR', OPSSVC_DIR . "/zeropen");
defined('OPSZEROPEN_TOKEN_DIR') or define('OPSZEROPEN_TOKEN_DIR', OPSTOKEN_DIR . "/zeropen");


function get_token($zpd_id)
{
	global $nidb;
	
	try {
		$sql = "select `token` from `cloud_platform`.`zeropen_device`";
		$sql.= " where `token` != '' and `zpd_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $zpd_id, \PDO::PARAM_INT);
		if($sth->execute() && ($row = $sth->fetch(\PDO::FETCH_ASSOC))) {
			return $row['token'];
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return '';
}

function make_token($zpd_id)
{
	global $nidb;

	$err = 0;
	do {
		$token = clouduuid();
		$newconfdir = OPSZEROPEN_TOKEN_DIR . "/{$token}";
		$err++;
	} while($err < 20 && file_exists($newconfdir));
	if($err >= 20) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '创建 token 失败！' . $token);
		return false;
	}

	// 保存 新的 token
	try {
		$sql = "select `token` from `cloud_platform`.`zeropen_device`";
		$sql.= " where `token` != '' and `zpd_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $zpd_id, \PDO::PARAM_INT);
		$sth->execute();
		$oldcfg = $sth->fetch(\PDO::FETCH_ASSOC);

		$sql = "update `cloud_platform`.`zeropen_device`";
		$sql.= " set `token` = ?";
		$sql.= " where `zpd_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $token, \PDO::PARAM_STR);
		$sth->bindParam(2, $zpd_id, \PDO::PARAM_INT);
		$ret = $sth->execute();

		$devicedir = OPSZEROPEN_DIR . "/device/{$zpd_id}";
		if($oldcfg) {
			$oldtoken = OPSZEROPEN_TOKEN_DIR . "/{$oldcfg['token']}";
			if(file_exists($oldtoken))
				unlink($oldtoken);
		}

		if(!is_dir($OPSZEROPEN_TOKEN_DIR))
			exec("mkdir -p " . OPSZEROPEN_TOKEN_DIR . " 2>&1", $output, $ret);
		exec("ln -s {$devicedir} {$newconfdir} 2>&1", $output, $ret);

		$ftoken = "{$devicedir}/token.dat";
		if(!is_dir($devicedir))
			exec("mkdir -p {$devicedir} 2>&1", $output, $ret);

		file_put_contents($ftoken, $token);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	return $token;
}

function make_sync($zpd_id, $name)
{
	global $nidb;
	// 如果需要客户同步配置，需要在 
	// OPSZEROPEN_DIR . "/device/{$zpd_id}/sync" 目录下
	// 创建 $name 文件，否则删除。
	
	$syncdir = OPSZEROPEN_DIR . "/device/{$zpd_id}/sync";
	if(!is_dir($syncdir))
		exec("mkdir -p {$syncdir} 2>&1", $output, $ret);

	file_put_contents("{$syncdir}/{$name}", time());
	
	// 通知客户端更新
	try {
		$sql = "select `device` from `cloud_platform`.`zeropen_device`";
		$sql.= " where `token` != '' and `zpd_id` = ?";
		$sth = $nidb->prepare($sql);
		$sth->bindParam(1, $zpd_id, \PDO::PARAM_INT);
		if($sth->execute() && ($row = $sth->fetch(\PDO::FETCH_ASSOC))) {
			$cmd = DATAEYE . " device set license_id12={$row['device']} task_signal=1";
			exec("{$cmd} > /dev/null 2>&1");
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
}

function clean_sync($zpd_id, $name)
{
	// 如果不需要客户同步配置，请删除
	// OPSZEROPEN_DIR . "/device/{$zpd_id}/sync" 目录下
	// 的指定 $name 文件，如果 $name 为空文本，则清空所有升级。

	$syncdir = OPSZEROPEN_DIR . "/device/{$zpd_id}/sync";
	if(!is_dir($syncdir)) return;
	
	if(empty($name))
		exec("rm -rf {$syncdir} 2>&1", $output, $ret);
	else
		exec("rm -f {$syncdir}/{$name} 2>&1", $output, $ret);
}

function isneed_sync($data, $key, $zpd_id, $name)
{
	// 用于自动检查标记，来判断是否需要更新操作
	if(isset($data[$key])) {
		if(intval($data[$key]))
			make_sync($zpd_id, $name); // 通知客户端更新
		else
			clean_sync($zpd_id, $name);// 删除客户端更新通知
	}
}

define('OPSZEROPEN_SYNC_NAME',	1);
define('OPSZEROPEN_SYNC_CFG',	2);
define('OPSZEROPEN_SYNC_LIC',	4);
define('OPSZEROPEN_SYNC_OPS',	8);

function ckmake_sync($zpd_id, $sync)
{
	$map = array('sysname.conf', 'panabit.conf', 'license.pac', 'shellops');

	$c = count($map);
	$i = 0;
	$v = 1;
	while($sync && $i < $c) {
		$syna = ($sync & $v);
		if($syna)
			make_sync($zpd_id, $map[$i]); // 通知客户端更新

		$v = $v << 1;
		$sync = $sync & (~$syna);
		$i++;
	}
	
	return $i;
}

function ckclean_sync($zpd_id, $sync)
{
	$map = array('sysname.conf', 'panabit.conf', 'license.pac', 'shellops');

	$c = count($map);
	$i = 0;
	$v = 1;
	while($sync && $i < $c) {
		$syna = ($sync & $v);
		if($syna)
			clean_sync($zpd_id, $map[$i]);// 删除客户端更新通知

		$v = $v << 1;
		$sync = $sync & (~$syna);
		$i++;
	}
	
	return $i;
}

function ckneed_sync($zpd_id, $sync)
{
	if(!$sync) {
		clean_sync($zpd_id, '');// 清除客户端所有需要同步通知
		return 1;
	}

	$map = array('sysname.conf', 'panabit.conf', 'license.pac', 'shellops');
	
	$c = count($map);
	$i = 0;
	$v = 1;
	while($sync && $i < $c) {
		$syna = ($sync & $v);
		if($syna)
			make_sync($zpd_id, $map[$i]); // 通知客户端更新
		else
			clean_sync($zpd_id, $map[$i]);// 删除客户端更新通知

		$v = $v << 1;
		$sync = $sync & (~$syna);
		$i++;
	}

	return $i;
}
/*
function logger($zpd_id, $date, $tag, $msg)
{
	$devicedir = OPSZEROPEN_DIR . "/{$zpd_id}";
	if(!file_exists($devicedir))
		exit;

	if(!is_dir("{$devicedir}/logger"))
		exec("mkdir -p {$devicedir}/logger 2>&1", $output, $ret);

	$flogger = "{$devicedir}/logger/" . date("Y-m-d", $date);
	if(($fp = fopen($flogger, 'a'))) {
		fwrite($fp, '[' . date("H:i:s", $date) . "] {$tag} {$msg}\n");
		fclose($fp);
	}
}
*/
function select($data)
{
	global $nidb, $user;


	$result = array(
		'rows'	=> array(),
		'total' => 0,
	);

	$values = array();
	$optional = array();
	// set order, limit, offset value
	format_list_arg($data, $optional);

	$where_str = "`disabled` < " . TB_ROW_DEL_VALUE . " and ";
	
	if(format_and_push($data, 'zpd_id', $optional, '', 'int', false) === false)
		$optional['zpd_id'] = 0;
	
	if($optional['zpd_id'])
		$where_str .= "`zpd_id` = {$optional['zpd_id']} and ";

	if(format_and_push($data, 'wkpid', $optional, '', 'int', false) === false)
		$wkpid = \cloud\apps\work\project\project_enable();
	else
		$wkpid = $optional['wkpid'];

	if($wkpid) 
		$where_str .= "`wkpid` = {$wkpid} and ";
	if(!is_supadmin($user->username)) {
		if($wkpid == 0) {
			$wkpids = \cloud\apps\work\project\get_work_for_user($user->username);
			if(!$wkpids) return $result;
			$where_str .= "`wkpid` in ({$wkpids}) and ";
		}
	}
	
	if(format_and_push($data, 'keyword', $optional, '', 'string', false)) {
		$where_str .= "(`name` like ? or `device` like ?) and ";
		array_push($values, "%" . $optional['keyword'] . "%");
		array_push($values, "%" . $optional['keyword'] . "%");
	}

	$order_map = array(
		"zpd_id",
		"wkpid",
		"name",
		"keyno",
		"clicnt",
		"mtime",
		"ctime"
	);

	$order = array();
	if(isset($optional['order'])) {
		foreach($optional['order'] as $op) {
			if(isset($op['column']) && isset($op['dir'])) {
				if(in_array($op['column'], $order_map)) {
					array_push($order, "p.`" . $op['column'] . "` " . ($op['dir']=='desc'?'desc':'asc'));
				}
			}
		}
	}

	if(count($order) > 0)
		$order_by = 'order by ' . implode(', ', $order);
	else
		$order_by = '';

	if($where_str != '')
		$where_str = substr("where $where_str", 0, -5);

	$sql = "select count(*) as `total` from cloud_platform.zeropen_device";
	$sql.= " $where_str";

	try {
		$sth = $nidb->prepare($sql);

		$sth->execute($values);

		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if(count($rows) > 0) {
		$result['total'] = $rows[0]->total;
	}

	$sql = "select * from cloud_platform.zeropen_device";
	$sql.= " $where_str $order_by limit ? offset ? ";

	array_push($values, $optional['limit']);
	array_push($values, $optional['offset']);

	try {
		$sth = $nidb->prepare($sql);
		$sth->execute($values);
		$rows = $sth->fetchAll(\PDO::FETCH_CLASS);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}


	$result['rows'] = $rows;

	if(isset($data['draw'])) $result['draw'] = $data['draw'];

	return $result;
}

function add($data)
{
	global $nidb, $user;


	if(isset($data['name']) == false || empty(($data['name'] = trim($data['name'])))) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '名称不能为空。');
		return false;
	}

	if(isset($data['sync']))
		$sync = intval($data['sync']);
	else
		$sync = OPSZEROPEN_SYNC_NAME;

	if(isset($data['cookie']) && !empty($cookie = trim($data['cookie']))) {
		if(preg_match("/^([a-zA-Z0-9-]{83})$/", $cookie, $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "83位系统编码格式不正确！");
			return false;
		}
	}
	else
		$cookie = '';

	if(isset($data['matchkeyno'])) {
		$matchkeyno = trim($data['matchkeyno']);
		if(preg_match("/^([a-zA-Z0-9-_]{12})$/", $matchkeyno, $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "软件编号格式不正确！");
			return false;
		}
	} else
		$matchkeyno = '';
	

	if(isset($data['device'])) {
		$data['device'] = trim($data['device']);
		if(preg_match("/^([a-zA-Z0-9-_]{12})$/", $data['device'], $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "软件编号格式不正确！");
			return false;
		}
	}

	$wkpid = \cloud\apps\work\project\project_enable();
	if(!$wkpid && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	$frmData = array(
		'wkpid'		=> $wkpid,
		'name'		=> $data['name'],
		'cookie'	=> $cookie,
		'matchkeyno'=> $matchkeyno,
		'sync'		=> OPSZEROPEN_SYNC_NAME,
		'ctime'		=> time()
	);
	
	if(isset($data['device']))
		$frmData['device'] = $data['device'];

	if(insert_data('cloud_platform`.`zeropen_device', $frmData) === false) {
		$errmsg = implode(' ', $nidb->errorInfo());
		set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
		return false;
	}
	$id = $nidb->lastInsertId();


	$confdir = OPSZEROPEN_DIR . "/device/{$id}";
	if(!is_dir($confdir))
		exec("mkdir -p {$confdir} 2>&1", $output, $ret);
	
	// 写入系统 名称
	$sysname = iconv('utf-8', 'gbk', $data['name']);
	file_put_contents("{$confdir}/sysname.conf", $sysname);

	if($cookie != '') {
		$cookiedir = OPSZEROPEN_DIR . "/cookie/{$cookie}";
		if(!is_dir($cookiedir))
			exec("mkdir -p {$cookiedir} 2>&1", $output, $ret);

		// 建立新的配置链接 ID 文件
		$idfile = "{$cookiedir}/{$id}";
		if(!file_exists($idfile))
			file_put_contents($idfile, time());

		// 更新 硬件编号
		file_put_contents("{$confdir}/cookie.dat", $cookie);
	}

	make_token($id);
	ckmake_sync($id, $sync);

	return $id;
}

function cpobj($data)
{
	global $nidb, $user;


	if(isset($data['zpd_id']) == false || ($zpd_id = intval($data['zpd_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '复制对象不正确。');
		return false;
	}

	if(isset($data['count']) == false || ($count = intval($data['count'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '复制对象不正确。');
		return false;
	}
	if($count > 1000) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '一次复制个数不能大于1000个。');
		return false;
	}


	$wkpid = \cloud\apps\work\project\project_enable();
	if(!$wkpid && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	// 检查复制对象
	try {
		$sql = "select * from `cloud_platform`.`zeropen_device`";
		$sql.= " where `zpd_id` = {$zpd_id}";
		if($wkpids)
			$sql.= " and `wkpid` in ({$wkpids})";
		$sth = $nidb->prepare($sql);

		$sth->execute();
		// $cnt = $sth->rowCount();
		$row = $sth->fetch(\PDO::FETCH_ASSOC);
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!$row) {
		set_errmsg(MSG_LEVEL_DEF, __function__, '指定的设备不存在。');
		return false;
	}

	$frmData = array(
		'wkpid'		=> $wkpid,
		'sync'		=> OPSZEROPEN_SYNC_NAME,
		'ctime'		=> time()
	);

	for($i = 0; $i < $count; $i++) {
		$frmData['name'] = $row['name'] . "_" . $i;

		if(insert_data('cloud_platform`.`zeropen_device', $frmData) === false) {
			$errmsg = implode(' ', $nidb->errorInfo());
			set_errmsg(MSG_LEVEL_DEF, __function__, $errmsg);
			return false;
		}
		$id = $nidb->lastInsertId();

		make_token($id);
		ckneed_sync($id, OPSZEROPEN_SYNC_NAME);
	}

	return $count;
}

function del($data)
{
	global $nidb, $user;


	if(isset($data['zpd_id']) == false || empty($data['zpd_id'] = trim($data['zpd_id']))) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定要删除的配置。');
		return false;
	}

	if(preg_match("/^([0-9,]{1,})$/", $data['zpd_id'], $match) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, "zpd_id 格式不正确！");
		return false;
	}

	$wkpids = \cloud\apps\work\project\project_enable();
	if(!$wkpids && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

/*
	// 删除路由配置
	try {
		$sql = "delete from `cloud_platform`.`zeropen_route`";
		$sql.= " where `zpd_id` in ({$data['zpd_id']})";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 删除LAN配置
	try {
		$sql = "delete from `cloud_platform`.`zeropen_lan`";
		$sql.= " where `zpd_id` in ({$data['zpd_id']})";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 删除WAN配置
	try {
		$sql = "delete from `cloud_platform`.`zeropen_wan`";
		$sql.= " where `zpd_id` in ({$data['zpd_id']})";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	// 删除网络接口配置
	try {
		$sql = "delete from `cloud_platform`.`zeropen_netif`";
		$sql.= " where `zpd_id` in ({$data['zpd_id']})";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	try {
		$sql = "select `device`, `cookie` from `cloud_platform`.`zeropen_device`";
		$sql.= " where `zpd_id` = {$data['zpd_id']}";
		$sth = $nidb->prepare($sql);
		if($sth->execute()) {
			$row = $sth->fetch(\PDO::FETCH_ASSOC);
			if(!$row) {
				$licfile = OPSZEROPEN_DIR . "/license/{$row['device']}.pac";
				if(file_exists($licfile)) unlink($licfile);
			}
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
*/

	// 删除设备配置
	try {
		$sql = "delete from `cloud_platform`.`zeropen_device`";
		$sql.= " where `zpd_id` in ({$data['zpd_id']})";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if(!is_dir(OPSHISTORY_DIR . "/zeropen/device"))
		exec("mkdir -p " . OPSHISTORY_DIR . "/zeropen/device 2>&1", $output, $ret);

	// 将配置移动至历史 device 回收目录
	// 已便日后分析
	$ids = explode(',', $data['zpd_id']);
	foreach($ids as $id) {
		clean_sync($id, '');

		$devicedir = OPSZEROPEN_DIR . "/device/{$id}";
		$newconfdir = OPSHISTORY_DIR . "/zeropen/device/{$id}";
		if(is_dir($newconfdir))
			exec("rm -rf {$newconfdir} 2>&1", $output, $ret);

		// 删除 token
		$ftoken = ${devicedir} . "/token.dat";
		if(file_exists($ftoken)) {
			$token = file_get_contents($ftoken);
			$ftoken = OPSZEROPEN_TOKEN_DIR . "/{$token}";
			if(file_exists($ftoken))
				unlink($ftoken);
		}

		// 删除 cookie 关联
		$fcookie = ${devicedir} . "/cookie.dat";
		if(file_exists($fcookie)) {
			if(($cookie = file_get_contents($fcookie))) {
				$fcookie = OPSZEROPEN_DIR . "/cookie/{$cookie}/{$id}";
				if(file_exists($fcookie))
					unlink($fcookie);
			}
		}

		// 移到 history 下面
		if(is_dir($devicedir))
			exec("mv {$devicedir} {$newconfdir} 2>&1", $output, $ret);
	}

	return true;
}

function save($data)
{
	global $nidb, $user;


	$wkpids = \cloud\apps\work\project\project_enable();
	if(!$wkpids && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	if(isset($data['zpd_id']) == false || ($data['zpd_id'] = intval($data['zpd_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定要删除的配置。');
		return false;
	}

	if(isset($data['name'])) {
		$name = trim($data['name']);
		if($name == '') {
			set_errmsg(MSG_LEVEL_ARG, __function__, "名称不能为空。");
			return false;
		}
	}

	if(isset($data['cookie'])) {
		$cookie = trim($data['cookie']);
		if($cookie != '' && preg_match("/^([a-zA-Z0-9-]{83})$/", $cookie, $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "83位系统编号不正确！");
			return false;
		}
	}
	
	if(isset($data['sync']))
		$sync = intval($data['sync']);
	else
		$sync = 0;

	if(isset($data['matchkeyno'])) {
		$matchkeyno = trim($data['matchkeyno']);
		if(preg_match("/^([a-zA-Z0-9-_]{12,})$/", $matchkeyno, $match) == false) {
			set_errmsg(MSG_LEVEL_ARG, __function__, "软件编号格式不正确！");
			return false;
		}
	}

	if(isset($data['unlock'])) 
		$unlock = intval($data['unlock']);
	else
		$unlock = 0;


	$old = false;
	// 获取原系统编号 cookie
	try {
		$sql = "select `name`, `cookie` from `cloud_platform`.`zeropen_device`";
		$sql.= " where `zpd_id` = {$data['zpd_id']}";
		$sth = $nidb->prepare($sql);
		if($sth->execute()) {
			$old = $sth->fetch(\PDO::FETCH_ASSOC);
		}
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}
	
	if($old && isset($data['name']) && $data['name'] != $old['name']) 
		$sync |= 1;

	// 保存设备配置
	try {
		$sql = "update `cloud_platform`.`zeropen_device`";
		$sql.= " set `mtime` = " . time();
		if(isset($data['cookie']))
			$sql.= " , `cookie` = '{$cookie}'";
		if(isset($data['name']))
			$sql.= " , `name` = ?";
		if(isset($data['matchkeyno']))
			$sql.= " , `matchkeyno` = ?";
		if($unlock)
			$sql.= " , `locked` = 0";
		if(isset($data['sync']))
			$sql.= " , `sync` = {$sync}";
		$sql.= " where `zpd_id` = {$data['zpd_id']}";
		$sth = $nidb->prepare($sql);

		$index = 1;
		if(isset($data['name']))
			$sth->bindParam($index++, $name, \PDO::PARAM_STR);
		if(isset($data['matchkeyno']))
			$sth->bindParam($index++, $matchkeyno, \PDO::PARAM_STR);

		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	if($old) {
		// 删除原来的配置链接 ID 文件
		if($old['cookie'] != '' && $cookie != $old['cookie']) {
			$idfile = OPSZEROPEN_DIR . "/cookie/{$old['cookie']}/{$data['zpd_id']}";
			if(file_exists($idfile))
				unlink($idfile);
		}
	}

	$confdir = OPSZEROPEN_DIR . "/device/{$data['zpd_id']}";
	if(!is_dir($confdir))
		exec("mkdir -p {$confdir} 2>&1", $output, $ret);

	if($sync & 1) {
		// 系统名称更新
		$sysname = iconv('utf-8', 'gbk', $data['name']);
		file_put_contents("{$confdir}/sysname.conf", $sysname);
	}

	// 更新 硬件编号的对应关系
	if(isset($data['cookie']) && $cookie != '') {
		$cookiedir = OPSZEROPEN_DIR . "/cookie/{$cookie}";
		if(!is_dir($cookiedir))
			exec("mkdir -p {$cookiedir} 2>&1", $output, $ret);

		// 建立新的配置链接 ID 文件
		$idfile = "{$cookiedir}/{$data['zpd_id']}";
		if(!file_exists($idfile))
			file_put_contents($idfile, time());

		// 更新 硬件编号
		file_put_contents("{$confdir}/cookie.dat", $cookie);
	}


	// 是否要解除与原来设备的绑定
	if($unlock) {
		make_token($data['zpd_id']);
		$lockfile = "{$confdir}/locked.dat";
		if(file_exists($lockfile))
			unlink($lockfile);
	}

	// 手动增加或删除匹配的设备
	if(isset($data['matchkeyno'])) {
		$lockfile = "{$confdir}/license.matchkeyno";
		if($matchkeyno != '') {
			file_put_contents($lockfile, $matchkeyno);
		}
		else {
			if(file_exists($lockfile))
				unlink($lockfile);
		}
	}

	// 解析同步指令
	if(isset($data['sync']))
		ckneed_sync($data['zpd_id'], $sync);

	return $ret;
}

function license_import($data)
{
	global $nidb, $user;


	$wkpids = \cloud\apps\work\project\project_enable();
	if(!$wkpids && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	if(isset($data['zpd_id']) == false || ($zpd_id = intval($data['zpd_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定配置。');
		return false;
	}

	if(!isset($_FILES['file'])){
		set_errmsg(MSG_LEVEL_ARG, __function__, "请选择要导入的授权文件!");
		return false;
	}

	$upfile = $_FILES['file'];
	if(file_exists($upfile['tmp_name']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, "找不到上传的license文件！");
		return false;
	}

	if($upfile['error'] > 0){
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "license文件上传失败！error：" . $upfile['error']);
		return false;
	}
	if(empty(ROOT_DIR) || is_dir(ROOT_DIR) == false){
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "应用程序目录不存在！error：" . ROOT_DIR);
		return false;
	}

	$fd = fopen($upfile['tmp_name'], 'r');
	if(!$fd) {
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "打开授权文件失败！");
		return false;
	}
	$keyno = fread($fd, 12);
	fclose($fd);

	if(!$keyno
	|| preg_match("/^([a-zA-Z0-9-]{12})$/", $keyno, $match) == false) {
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "授权文件不正确！");
		return false;
	}

	$confdir = OPSZEROPEN_DIR . "/device/{$data['zpd_id']}";
	$licfile = "{$confdir}/license.pac";
	if(!is_dir($confdir))
		exec("mkdir -p {$confdir} 2>&1", $output, $ret);

	// 将授权文件存放到 device/$data['zpd_id'] 目录下
	if(file_exists($licfile)) unlink($licfile);
	if(!move_uploaded_file($upfile['tmp_name'], $licfile)) {
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "保存授权文件失败！");
		return false;
	}

	$now = time();
	// 更新授权
	try {
		$sql = "update `cloud_platform`.`zeropen_device`";
		$sql.= " set `device` = '{$keyno}'";
		$sql.= " , `mtime` = {$now}";
		$sql.= " where `zpd_id` = {$data['zpd_id']}";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 检查是否需要更新
	make_sync($data['zpd_id'], 'license.pac');

	return true;
}

function config_import($data)
{
	global $nidb, $user;


	$wkpids = \cloud\apps\work\project\project_enable();
	if(!$wkpids && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	if(isset($data['zpd_id']) == false || ($zpd_id = intval($data['zpd_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定配置。');
		return false;
	}

	if(!isset($_FILES['file'])){
		set_errmsg(MSG_LEVEL_ARG, __function__, "请选择要导入的PA配置文件!");
		return false;
	}

	$upfile = $_FILES['file'];
	if(file_exists($upfile['tmp_name']) == false) {
		set_errmsg(MSG_LEVEL_ARG, __function__, "找不到上传的PA配置文件！" );
		return false;
	}

	if($upfile['error'] > 0){
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "PA配置文件上传失败！error：" . $upfile['error']);
		return false;
	}
	if(empty(ROOT_DIR) || is_dir(ROOT_DIR) == false){
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "应用程序目录不存在！error：" . ROOT_DIR);
		return false;
	}

	$fd = fopen($upfile['tmp_name'], 'r');
	if(!$fd) {
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "打开上传的配置文件失败！");
		return false;
	}
	$good = 0;
	while(!feof($fd)) {
		if(($buf = fgets($fd)) === false)
			break;
		if(!empty($buf = trim($buf, " \t\n\r\0\x0B"))) {
			$good = ($buf == 'ENDCONFIG');
			//if(!$good && $buf != "") {

			//}
		}
	}
	fclose($fd);
	if(!$good) {
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "PA配置文件的内容格式不正确！结尾行缺少“ENDCONFIG”标识。");
		return false;
	}

	// 将PA配置文件存放到conf目录中
	$confdir = OPSZEROPEN_DIR . "/device/{$data['zpd_id']}";
	$file = "{$confdir}/panabit.conf";
	if(!is_dir($confdir))
		exec("mkdir -p {$confdir} 2>&1", $output, $ret);
	if(!move_uploaded_file($upfile['tmp_name'], $file)) {
		unlink($upfile['tmp_name']);
		set_errmsg(MSG_LEVEL_ARG, __function__, "保存PA配置文件失败！");
		return false;
	}

	$now = time();
	// 更新PA配置
	try {
		$sql = "update `cloud_platform`.`zeropen_device`";
		$sql.= " set `mtime` = {$now}, `cfgsync` = " . $data['cfgsync'];
		$sql.= " where `zpd_id` = {$data['zpd_id']}";
		$sth = $nidb->prepare($sql);
		$ret = $sth->execute();
	}
	catch (\NiDBException $e) {
		set_errmsg(MSG_LEVEL_EXP, __function__, $e->getMessage());
		return false;
	}

	// 如果需要客户同步配置，需要在 
	// OPSZEROPEN_DIR . "/device/{$zpd_id}/sync" 目录下
	// 增加 panabit.conf 文件，否则删除。
	// 检查是否需要更新
	isneed_sync($data, 'cfgsync', $data['zpd_id'], 'panabit.conf');

	return true;
}

function config_get($data)
{
	global $nidb, $user;


	$wkpids = \cloud\apps\work\project\project_enable();
	if(!$wkpids && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		//set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return '';
	}

	if(isset($data['zpd_id']) == false || ($zpd_id = intval($data['zpd_id'])) <= 0) {
		//set_errmsg(MSG_LEVEL_ARG, __function__, '请指定配置。');
		return '';
	}

	// 将PA配置文件存放到conf目录中
	$file = OPSZEROPEN_DIR . "/device/{$data['zpd_id']}/panabit.conf";
	if(!file_exists($file))
		return '';

	if(!($text = file_get_contents($file)))
		$text = '';
	else
		$text = iconv('gbk', 'utf-8//translit', $text);

/*
	if(($fp = fopen($file, 'r'))) {
		while(!feof($fp)) {
			if(($line = fgets($fp)) === false)
				break;
			echo iconv('gbk', 'utf-8', $line);
		}
		fclose($fp);
	}
*/

	return $text;
}

function config_set($data)
{
	global $nidb, $user;


	$wkpids = \cloud\apps\work\project\project_enable();
	if(!$wkpids && !\cloud\apps\work\project\in_work_project(ROLE_ADMIN)) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '您的权限不足。');
		return false;
	}

	if(isset($data['zpd_id']) == false || ($zpd_id = intval($data['zpd_id'])) <= 0) {
		set_errmsg(MSG_LEVEL_ARG, __function__, '请指定配置。');
		return false;
	}

	if(isset($data['text']) == false || empty($text = trim($data['text'])))
		$text = '';
	else 
		$text = iconv('utf-8', 'gbk', $text);
	
	if(strpos($text, 'ENDCONFIG') === false)
		$text.= "\nENDCONFIG";

	// 将PA配置文件存放到conf目录中
	$confdir = OPSZEROPEN_DIR . "/device/{$data['zpd_id']}";
	$file = "{$confdir}/panabit.conf";
	if(!is_dir($confdir))
		exec("mkdir -p {$confdir} 2>&1", $output, $ret);

	if(!($ret = file_put_contents($file, $text)))
		set_errmsg(MSG_LEVEL_ARG, __function__, '保存配置到文件失败。');

	// 如果需要客户同步配置，需要在 
	// OPSZEROPEN_DIR . "/device/{$zpd_id}/sync" 目录下
	// 增加 panabit.conf 文件，否则删除。
	// 检查是否需要更新
	isneed_sync($data, 'cfgsync', $data['zpd_id'], 'panabit.conf');

	return $ret;
}

