#!/bin/sh

# check openssl 
openssl=/usr/bin/openssl
[ ! -f ${openssl} ] && openssl=/usr/local/bin/openssl
[ ! -f ${openssl} ] && openssl=/bin/openssl

if [ ! -f ${openssl} ]; then
        echo "not fund openssl."
        exit 1
fi

CERT_DIR=${MY_CONF_DIR}/cert
[ ! -d ${CERT_DIR} ] && mkdir -p ${CERT_DIR}
SSL_KEY_LEN=2048
CA_DAYS=10000
CERT_DAYS=375

subj="/C=CN/ST=Beijing/L=Beijing/O=Panabit"

if [ ! -f ${CERT_DIR}/ssl.cnf ]; then
	if [ ! -f ${MY_APP_DIR}/bin/action/ssl.cnf ]; then
		echo "not found ssl.cnf, now copy ${MY_APP_DIR}/bin/action/ssl.cnf"	
		if [ ! -f /etc/ssl/openssl.cnf ]; then
			echo "not found default /etc/ssl/openssl.cnf"
		else
			echo "you can: cp -f /etc/ssl/openssl.cnf ${MY_APP_DIR}/bin/action/ssl.cnf"
		fi
		exit 1;
	fi
fi
cp -f ${MY_APP_DIR}/bin/action/ssl.cnf ${CERT_DIR}/ssl.cnf

[ ! -f ${CERT_DIR}/certindex.txt ] && touch ${CERT_DIR}/certindex.txt
[ ! -f ${CERT_DIR}/serial ] && echo 100003 > ${CERT_DIR}/serial
[ ! -d ${CERT_DIR}/certs ] && mkdir -p ${CERT_DIR}/certs

create_ca_cert()
{
	echo "CA certificate (FQDN must be different from server/client)"
	${openssl} genrsa -out ca.key ${SSL_KEY_LEN}
	${openssl} req -new -x509 -config ${CERT_DIR}/ssl.cnf -days ${CA_DAYS} -key ca.key -subj "${subj}/CN=Panabit Root CA" -out ca.crt
}

create_server_cert()
{
	echo "server certificate (for multiple domains, change subjectAltName to: DNS:example.com,DNS:www.example.com)"
	openssl req -newkey rsa:${SSL_KEY_LEN} -config ${CERT_DIR}/ssl.cnf -nodes -keyout server.key -subj "${subj}/CN=localhost" -out server.csr
	echo "subjectAltName=DNS:localhost" > ext.cnf
	openssl x509 -sha256 -req -extfile ext.cnf -days ${CERT_DAYS} -in server.csr -CA ca.crt -CAkey ca.key -CAcreateserial -out server.crt
	rm -f ext.cnf
}

crete_client_cert()
{
	echo "# CREATE CLIENT CERT:"
	openssl req -newkey rsa:${SSL_KEY_LEN} -config ${CERT_DIR}/ssl.cnf -nodes -keyout client.key -subj "${subj}/CN=client" -out client.csr
	openssl x509 -req -days ${CERT_DAYS} -in client.csr -CA ca.crt -CAkey ca.key -CAcreateserial -out client.crt
	openssl pkcs12 -export -clcerts -in client.crt -inkey client.key -out client.p12
	openssl pkcs12 -in client.p12 -out client.pem -clcerts
}

clean_cert()
{
	rm -rf ${CERT_DIR}
}

help_cert()
{
	echo "appctrl genkey ca|server|client|all|clean|help"	
}

cd ${CERT_DIR}
case ${1} in
	"ca")
		create_ca_cert
		create_server_cert
	;;
	"server")
		create_server_cert
	;;
	"client")
		crete_client_cert
	;;
	"all")
		create_ca_cert
		create_server_cert
		crete_client_cert
	;;
	"clean")
		clean_cert	
	;;
	"help")
		help_cert
	;;
	*)
		help_cert
		cd -
		exit 1
	;;
esac
cd -

